package cn.kdan.pdf.backend.core.client;

import cn.kdan.pdf.backend.core.constant.ComPdfKitConstant;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.*;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.*;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.ObjectUtils;
import org.springframework.web.client.RestTemplate;
import utils.JsonUtils;
import utils.RedisUtils;

import java.io.File;
import java.time.Duration;
import java.util.HashMap;
import java.util.Map;

/**
 * Created with IDEA
 *
 * @author tangxiangan
 * @date 2022/6/4 9:46
 * Description
 */
@Slf4j
public class ComPdfKitClient {
    /**
     * 地址
     */
    private final String address;
    private final RestTemplate restTemplate;
    /**
     * 基础认证
     */
    private String basicAuth;
    private RedisUtils<String, String> redisUtils;

    private ComPdfKitClient(String address, String projectKey, String secretKey, Duration readTimeout, Duration connectTimeout) {
        if (!address.endsWith("/")) {
            address += "/";
        }
        this.address = address;
        this.restTemplate = new RestTemplateBuilder()
                .setReadTimeout(readTimeout)
                .setConnectTimeout(connectTimeout)
                .setBufferRequestBody(false)
                .build();
        this.basicAuth = basicAuth(projectKey, secretKey);
    }

    public static ComPdfKitClientBuilder builder() {
        return new ComPdfKitClientBuilder();
    }

    public void basic(String projectKey, String secretKey) {
        this.basicAuth = basicAuth(projectKey, secretKey);
    }

    private HttpHeaders basicHeaders() {
        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization", "Bearer " + this.basicAuth);
        return headers;
    }

    /**
     * 获取token
     *
     * @param projectKey 项目key
     * @param secretKey  秘钥
     * @return String
     */
    public String basicAuth(String projectKey, String secretKey) {
//        if(!StringUtils.isEmpty(redisUtils.get(ComPDFKitConstant.COM_PDF_Kit_TOKEN))){
//            return redisUtils.get(ComPDFKitConstant.COM_PDF_Kit_TOKEN);
//        }
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        Map<String, String> tokenParam = new HashMap<>();
        tokenParam.put("publicKey", projectKey);
        tokenParam.put("secretKey", secretKey);
        ResponseEntity<ComPdfKitResult<ComPdfKitOauthResult>> responseEntity;
        ParameterizedTypeReference<ComPdfKitResult<ComPdfKitOauthResult>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<ComPdfKitOauthResult>>() {};
        try {
            responseEntity = restTemplate.exchange(
                    address.concat(ComPdfKitConstant.API_V1_OAUTH_TOKEN),
                    HttpMethod.POST,
                    new HttpEntity<>(JsonUtils.getJsonString(tokenParam), headers),
                    typeRef
            );
        } catch (Exception e) {
            log.error(ComPdfKitConstant.EXCEPTION_MSG_GET_TOKEN_FAIL + "{}", e.getMessage());
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_GET_TOKEN_FAIL);
        }
        if (responseEntity.getStatusCode() != HttpStatus.OK || ObjectUtils.isEmpty(responseEntity.getBody())) {
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_GET_TOKEN_FAIL);
        }
        // 设置过期
        // redisUtils.set(ComPDFKitConstant.COM_PDF_Kit_TOKEN, token, responseEntity.getBody().get(ComPDFKitConstant.EXPIRES_IN).asLong()-20L);
        return responseEntity.getBody().getData().getAccessToken();
    }

    /**
     * 创建任务
     *
     * @param executeTypeUrl 任务执行类型
     * @return CreateTaskResult
     */
    public CreateTaskResult createTask(String executeTypeUrl) {
        String url = address.concat(ComPdfKitConstant.API_V1_CREATE_TASK).replace("{executeTypeUrl}", executeTypeUrl);
        ResponseEntity<ComPdfKitResult<CreateTaskResult>> response;
        ParameterizedTypeReference<ComPdfKitResult<CreateTaskResult>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<CreateTaskResult>>() {};
        try {
            response = restTemplate.exchange(
                    url,
                    HttpMethod.GET,
                    new HttpEntity<>(basicHeaders()),
                    typeRef
            );
        } catch (Exception e) {
            log.error(ComPdfKitConstant.EXCEPTION_MSG_CREATE_TASK_FAIL + "{}", e.getMessage());
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_CREATE_TASK_FAIL + e.getMessage());
        }
        if (response.getStatusCode() != HttpStatus.OK || ObjectUtils.isEmpty(response.getBody() )){
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_CREATE_TASK_FAIL);
        }
        return response.getBody().getData();
    }

    /**
     * 上传文件
     *
     * @param file     文件
     * @param taskId   任务id
     * @param password 密码
     * @return UploadFileResult
     */
    public UploadFileResult uploadFile(File file, String taskId, String password) throws Exception {
        log.info("开始上传文件，taskId：{},password:{}",taskId,password);
        String url = address.concat(ComPdfKitConstant.API_V1_UPLOAD_FILE);
        MultiValueMap<String, Object> param = new LinkedMultiValueMap<>();;
        FileSystemResource fs = new FileSystemResource(file);
        param.add("file", fs);
        param.add("taskId", taskId);
        param.add("password", password);
        HttpHeaders headers = basicHeaders() ;
        headers.setContentType(MediaType.MULTIPART_FORM_DATA);
        ResponseEntity<ComPdfKitResult<UploadFileResult>> response = null;
        ParameterizedTypeReference<ComPdfKitResult<UploadFileResult>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<UploadFileResult>>() {};
        try {
            response = restTemplate.exchange(
                    url,
                    HttpMethod.POST,
                    new HttpEntity<>(param, headers),
                    typeRef
            );
        } catch (Exception e) {
            log.error(ComPdfKitConstant.EXCEPTION_MSG_UPLOAD_FILE_FAIL + "{}", e.getMessage());
            throw new Exception(ComPdfKitConstant.EXCEPTION_MSG_UPLOAD_FILE_FAIL + e.getMessage());
        }finally {
            try {
                file.delete();
            }catch (Exception e){
                log.error("删除文件失败；{}",e.getMessage());
            }
        }

        if (response.getStatusCode() != HttpStatus.OK || ObjectUtils.isEmpty(response.getBody()) || !CommonConstant.SUCCESS_CODE.equals(response.getBody().getCode()) || ObjectUtils.isEmpty(response.getBody())) {
            throw new Exception(ComPdfKitConstant.EXCEPTION_MSG_UPLOAD_FILE_FAIL+ response.getBody().getMsg());
        }
        return response.getBody().getData();
    }

    /**
     * 执行任务转档
     *
     * @param taskId 任务id
     * @return 任务id
     */
    public CreateTaskResult executeTask(String taskId){
        log.info("开始执行任务转档，taskId：{}",taskId);
        String url = address.concat(ComPdfKitConstant.API_V1_EXECUTE_TASK).concat("?taskId=").concat(taskId);
        ResponseEntity<ComPdfKitResult<CreateTaskResult>> response;
        ParameterizedTypeReference<ComPdfKitResult<CreateTaskResult>> result = new ParameterizedTypeReference<ComPdfKitResult<CreateTaskResult>>(){};
        try{
            response = restTemplate.exchange(
                    url,
                    HttpMethod.GET,
                    new HttpEntity<>(JsonUtils.getJsonString(taskId),basicHeaders()),
                    result
            );
        }catch (Exception e){
            log.error(ComPdfKitConstant.EXCEPTION_MSG_EXECUTE_TASK_FAIL + "{}", e.getMessage());
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_EXECUTE_TASK_FAIL + e.getMessage());
        }
        if(response.getStatusCode() != HttpStatus.OK || ObjectUtils.isEmpty(response.getBody()) || !CommonConstant.SUCCESS_CODE.equals(response.getBody().getCode()) || ObjectUtils.isEmpty(response.getBody())){
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_EXECUTE_TASK_FAIL+response.getBody());
        }
        return response.getBody().getData();
    }

    /**
     * 查询任务文件状态
     *
     * @param taskId 任务id
     * @return QueryTaskInfoResult
     */
    public QueryTaskInfoResult queryTaskInfo(String taskId){
        log.info("开始查询转档状态，taskId：{}",taskId);
        String url = address.concat(ComPdfKitConstant.API_V1_TASK_INFO).concat("?taskId=").concat(taskId);
        ResponseEntity<ComPdfKitResult<QueryTaskInfoResult>> response;
        ParameterizedTypeReference<ComPdfKitResult<QueryTaskInfoResult>> result = new ParameterizedTypeReference<ComPdfKitResult<QueryTaskInfoResult>>(){};
        try{
            response = restTemplate.exchange(
                    url,
                    HttpMethod.GET,
                    new HttpEntity<>(JsonUtils.getJsonString(taskId),basicHeaders()),
                    result
            );
        }catch (Exception e){
            log.error(ComPdfKitConstant.EXCEPTION_MSG_TASK_INFO_FAIL + "{}", e.getMessage());
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_TASK_INFO_FAIL + e.getMessage());
        }
        if(response.getStatusCode() != HttpStatus.OK || ObjectUtils.isEmpty(response.getBody()) || !CommonConstant.SUCCESS_CODE.equals(response.getBody().getCode()) || ObjectUtils.isEmpty(response.getBody().getData())){
            throw new BackendRuntimeException(ComPdfKitConstant.EXCEPTION_MSG_TASK_INFO_FAIL+response.getBody());
        }
        log.info("查询状态成功：{}",JsonUtils.getJsonString(response.getBody().getData()));
        return response.getBody().getData();
    }

    public static class ComPdfKitClientBuilder {
        private String projectKey;
        private String secretKey;
        private String address;
        private Duration readTimeout;
        private Duration connectTimeout;

        private ComPdfKitClientBuilder() {
        }

        public ComPdfKitClientBuilder readTimeout(Duration readTimeout) {
            this.readTimeout = readTimeout;
            return this;
        }

        public ComPdfKitClientBuilder connectTimeout(Duration connectTimeout) {
            this.connectTimeout = connectTimeout;
            return this;
        }

        public ComPdfKitClientBuilder basic(String address, String projectKey, String secretKey) {
            this.address = address;
            this.projectKey = projectKey;
            this.secretKey = secretKey;
            return this;
        }

        public ComPdfKitClient build() {
            if (this.readTimeout == null) {
                this.readTimeout = Duration.ofSeconds(60*5);
            }
            if (this.connectTimeout == null) {
                this.connectTimeout = Duration.ofSeconds(60*5);
            }
            return new ComPdfKitClient(this.address, this.projectKey, this.secretKey, this.readTimeout, this.connectTimeout);
        }

    }


}
