package cn.kdan.pdf.backend.core.config.oauth;

import cn.kdan.pdf.backend.core.service.AuthService;
import cn.kdan.pdf.backend.core.service.impl.CustomAuthenticationCodeServicesImpl;
import cn.kdan.pdf.backend.core.service.impl.CustomClientDetailsServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.oauth2.config.annotation.configurers.ClientDetailsServiceConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configuration.AuthorizationServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableAuthorizationServer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerEndpointsConfigurer;
import org.springframework.security.oauth2.config.annotation.web.configurers.AuthorizationServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.ClientDetailsService;
import org.springframework.security.oauth2.provider.token.DefaultTokenServices;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.security.oauth2.provider.token.store.redis.RedisTokenStore;

/**
 * 配置授权认证服务类
 *
 * @author tangxiangan
 */
@EnableAuthorizationServer //授权认证中心
@Configuration
public class AuthorizationServerConfig extends AuthorizationServerConfigurerAdapter {

    /**
     * 权限验证控制器
     */
    @Autowired
    private AuthenticationManager authenticationManager;
    /**
     * redis连接工厂
     */
    @Autowired
    private LettuceConnectionFactory redisConnectionFactory;

    @Autowired
    private AuthService authService;

    /**
     * 配置AccessToken的存储方式：此处使用Redis存储
     * Token的可选存储方式
     * 1、InMemoryTokenStore
     * 2、JdbcTokenStore
     * 3、JwtTokenStore
     * 4、RedisTokenStore
     * 5、JwkTokenStore
     */
    @Bean
    public TokenStore tokenStore() {
        return new RedisTokenStore(redisConnectionFactory);
    }

    /**
     * 用来配置令牌端点(Token Endpoint)的安全约束.
     * 针对授权服务器的安全，我们干了两个事情，首先打开了验证Token的访问权限
     * 然后允许ClientSecret明文方式保存并且可以通过表单提交（而不仅仅是Basic Auth方式提交）
     *
     * @param security
     * @throws Exception
     */
    @Override
    public void configure(AuthorizationServerSecurityConfigurer security) throws Exception {
        //允许所有资源服务器访问公钥端点（/oauth/token_key） //只允许验证用户访问令牌解析端点（/oauth/check_token）
        security.tokenKeyAccess("permitAll()")
                .checkTokenAccess("permitAll()")
                .allowFormAuthenticationForClients();// 允许客户端发送表单来进行权限认证来获取令牌
    }

    /**
     * 用来配置客户端详情服务（ClientDetailsService），
     * 客户端详情信息在这里进行初始化，  数据库在进行client_id 与 client_secret验证时   会使用这个service进行验证
     *
     * @param clients
     * @throws Exception
     */
    @Override
    public void configure(ClientDetailsServiceConfigurer clients) throws Exception {
        clients.withClientDetails(clientDetails());

    }

    /**
     * oauth的客户端凭证校验是通过ClientDetailsService来实现的
     * oauth默认为我们提供了InMemoryClientDetailsService和JdbcClientDetailsService，使用自己实现的ClientDetailsService。
     * 这里我们需要加上@Primary把该类当成是主类，因为在@EnableAuthorizationServer注解中会引入配置类ClientDetailsServiceConfiguration，该配置类会创建一个默认的ClientDetailsService，如果不加@Primary会导致循环
     *
     * @return
     */
    @Bean
    public ClientDetailsService clientDetails() {
        return new CustomClientDetailsServiceImpl(authService);
    }


    /**
     * 用来配置授权（authorizatio）以及令牌（token）的访问端点和令牌服务   核心配置  在启动时就会进行配置
     *
     * @param endpoints
     * @throws Exception
     */
    @Override
    public void configure(AuthorizationServerEndpointsConfigurer endpoints) throws Exception {
        //开启密码授权类型
        endpoints.authenticationManager(authenticationManager);
        //我们如果自己不配置AuthorizationCodeServices，那么默认实现就是InMemoryAuthorizationCodeServices授权码存在内存中
        //这里使用我们自定义的服务让授权码存在redis中
        endpoints.authorizationCodeServices(new CustomAuthenticationCodeServicesImpl(redisConnectionFactory));
        //配置token存储方式
        endpoints.tokenServices(defaultTokenServices());
    }


    /**
     * <p>注意，自定义TokenServices的时候，需要设置@Primary，否则报错，</p>
     * 自定义的token
     * 认证的token是存到redis里的
     *
     * @return
     */
    @Primary
    @Bean
    public DefaultTokenServices defaultTokenServices() {
        DefaultTokenServices tokenServices = new DefaultTokenServices();
        tokenServices.setTokenStore(tokenStore());
        tokenServices.setSupportRefreshToken(true);
        // token有效期自定义设置，modify 过期时间为15天
        tokenServices.setAccessTokenValiditySeconds(60 * 60 * 24 * 15);
        // refresh_token默认30天
        tokenServices.setRefreshTokenValiditySeconds(60 * 60 * 24 * 30);
        return tokenServices;
    }
}
