package cn.kdan.pdf.backend.core.config.oauth;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;
import pojo.CustomUserDetails;
import pojo.ResultMap;
import utils.JsonUtils;

/**
 * 接口认证类，定义了认证方法authenticate() 。
 * AbstractUserDetailsAuthenticationProvider 为认证抽象类，实现了接口 AuthenticationProvider 定义的认证方法 authenticate()。还定义了抽象方法 retrieveUser() 用于查询数据库用户信息，以及抽象方法 additionalAuthenticationChecks() 用作额外的身份验证检查。
 *
 * @author tangxiangan
 */
@Component("loginAuthenticationProvider")
public class LoginAuthenticationProvider extends AbstractUserDetailsAuthenticationProvider {

    @Autowired
    private UserService userService;

    @Autowired
    private UserDetailsService userDetailsService;

    @Autowired
    private PasswordEncoder passwordEncoder;

    /**
     * 实现密码是否匹配的检测
     */
    @Override
    protected void additionalAuthenticationChecks(UserDetails userDetails, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
        if (userDetails instanceof CustomUserDetails) {
            CustomUserDetails customUserDetails = (CustomUserDetails) userDetails;
            if (authentication.getCredentials() != null) {
                Members user = userService.getById(customUserDetails.getId());
                if (user != null) {
                    //判断密码是否正确
                    String presentedPassword = authentication.getCredentials().toString();
                    if (passwordEncoder.matches(presentedPassword, userDetails.getPassword())) {
                        //验证通过
                        return;
                    }
                }
            }
        }
        throw new BadCredentialsException(JsonUtils.getJsonString(new ResultMap<>(AuthConstant.EXCEPTION_CODE_USER_PASSWORD_ERROR, AuthConstant.EXCEPTION_MSG_USER_PASSWORD_ERROR)));
    }

    /**
     * authenticate方法里，会先从userCache里获取UserDetails
     * 如果userCache里拿到的是null，就进行retrieveUser，也就是根据账号、密码到数据库里去取用户信息，
     * 手动加载user
     */
    @Override
    protected UserDetails retrieveUser(String s, UsernamePasswordAuthenticationToken usernamePasswordAuthenticationToken) throws AuthenticationException {
        UserDetails userDetail = userDetailsService.loadUserByUsername(s);
        if (userDetail == null) {
            throw new InternalAuthenticationServiceException(JsonUtils.getJsonString(new ResultMap<>(AuthConstant.EXCEPTION_CODE_USER_PASSWORD_ERROR, AuthConstant.EXCEPTION_MSG_USER_PASSWORD_ERROR)));
        } else {
            return userDetail;
        }
    }

}
