package cn.kdan.pdf.backend.core.config.oauth;


import cn.kdan.pdf.backend.core.pojo.oauth2.AesPasswordEncoder;
import cn.kdan.pdf.backend.core.pojo.oauth2.LoginPasswordEncoder;
import cn.kdan.pdf.backend.core.service.AuthService;
import constant.CommonConstant;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;

/**
 * 是默认情况下spring security的http配置 优于ResourceServerConfigurerAdapter的配置
 */
@Configuration
@EnableWebSecurity
@EnableAutoConfiguration(exclude = {org.springframework.boot.autoconfigure.security.servlet.SecurityAutoConfiguration.class})
@Order(1)
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    /**
     * 密码验证处理器
     */
    @Autowired
    private LoginAuthenticationProvider loginAuthenticationProvider;

    @Autowired
    private LoginAuthenticationSuccessHandler loginAuthenticationSuccessHandler;

    @Autowired
    private LoginAuthenticationFailHandler loginAuthenticationFailHandler;

    @Autowired
    private LogoutAuthenticationSuccessHandler logoutAuthenticationSuccessHandler;

    @Value("${httpMatchers.request}")
    private String httpRequestMatchers;
    @Value("${httpMatchers.web}")
    private String httpWebMatchers;


    /**
     * 所有security的加密都会使用该加密器
     */
    @Bean
    public PasswordEncoder passwordEncoder() {
        //密码加密问题，默认使用BCryptPasswordEncoder，在注册时是否也需要进行MD5PasswordEncoder加密
        return new LoginPasswordEncoder();
    }


    /**
     * AuthenticationManager 为认证管理接口类，其定义了认证方法authenticate()
     * 这一步的配置是必不可少的，否则SpringBoot会自动配置一个AuthenticationManager,覆盖掉内存中的用户
     */
    @Override
    @Bean
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    /**
     * spring security设置
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.requestMatchers().antMatchers(httpRequestMatchers.split(CommonConstant.STRING_SIGN_COMMA));
        http.authorizeRequests().antMatchers(httpRequestMatchers.split(CommonConstant.STRING_SIGN_COMMA)).permitAll();
        // CSRF禁用，因为不使用session
        http.csrf().disable();
        http.formLogin()
                .successHandler(loginAuthenticationSuccessHandler)
                .failureHandler(loginAuthenticationFailHandler)
                .permitAll();//允许所有用户都有权限访问登录页面
        http.logout()
                .logoutSuccessHandler(logoutAuthenticationSuccessHandler)
                .permitAll();
        //图形验证码校验器
        //      http.addFilterBefore(new LoginAuthenticationProcessingFilter("/login", "/login?error", authService), UsernamePasswordAuthenticationFilter.class);
    }

    /**
     * 权限管理器  AuthorizationServerConfigurerAdapter认证中心需要的AuthenticationManager需要
     * 所有 UserDetails 相关的它都管，包含 PasswordEncoder 密码等
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) { // @formatter:off
        loginAuthenticationProvider.setHideUserNotFoundExceptions(false);
        //目的是为了前端获取数据时获取到整个form-data的数据,提供验证器
        auth.authenticationProvider(loginAuthenticationProvider);
    }

    @Override
    public void configure(WebSecurity web) {
        web.ignoring()
                .antMatchers(httpWebMatchers.split(CommonConstant.STRING_SIGN_COMMA));
    }
}
