package cn.kdan.pdf.backend.core.controller;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.backend.core.enums.VerifyTypeEnum;
import cn.kdan.pdf.backend.core.params.AppLogoutParam;
import cn.kdan.pdf.backend.core.params.RelationParam;
import cn.kdan.pdf.backend.core.pojo.app.LoginInfoVo;
import cn.kdan.pdf.backend.core.pojo.oauth2.TokenPOJO;
import cn.kdan.pdf.backend.core.service.AuthService;
import constant.CommonConstant;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import pojo.AppRequestParam;
import pojo.AppResultMap;
import pojo.ResultMap;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.validation.constraints.NotBlank;
import java.io.IOException;


/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/auth")
public class AuthController {

    @Resource
    private AuthService authService;


    /**
     * 获取token
     *
     * @param code 授权码
     */
    @GetMapping("/getToken")
    public ResultMap<TokenPOJO> getToken(@NotBlank(message = "授权码不能为空") @RequestParam String code) {
        TokenPOJO token = authService.getToken(code);
        if (token == null) {
            return new ResultMap<>(AuthConstant.EXCEPTION_CODE_GET_TOKEN_FAIL, AuthConstant.EXCEPTION_MSG_GET_TOKEN_FAIL);
        } else {
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, token);
        }
    }

    /**
     * 获取图片验证码
     *
     * @param width     图片的宽度
     * @param height    图片的长度
     * @param codeCount 验证码字符个数
     * @param lineCount 干扰线的数量
     */
    @GetMapping("/getImageCode")
    public void getImageCode(HttpServletResponse response, HttpSession session, @RequestParam(required = false) String type, @RequestParam int width,
                             @RequestParam int height, @RequestParam int codeCount, @RequestParam int lineCount) throws IOException {
        authService.generateImageCode(response, session, type, width, height, codeCount, lineCount);
    }

    /**
     * 获取手机验证码或者邮箱验证邮件
     * @param action 验证动作 CaptchaActionEnum
     * @param type 验证类型 0邮箱 1手机
     * @param receiver 接收者 （邮箱或者手机号）
     * @return Boolean
     */
    @GetMapping("/getVerifyCode")
    public ResultMap<Boolean> getVerifyCode(@RequestParam String action,@RequestParam Integer type,@NotBlank(message = "接收者不能为空") @RequestParam String receiver) {
        CaptchaActionEnum actionEnum = CaptchaActionEnum.getEnumByValue(action);
        if (actionEnum == null) {
            return new ResultMap<>(CommonConstant.ERROR, AuthConstant.EXCEPTION_MSG_CODE_ACTION_ERROR, false);
        }
        VerifyTypeEnum verifyTypeEnum = VerifyTypeEnum.getEnumByCode(type);
        if (verifyTypeEnum == null) {
            return new ResultMap<>(CommonConstant.ERROR, AuthConstant.EXCEPTION_MSG_CODE_TYPE_ERROR, false);
        }
        if (authService.getVerifyCode(actionEnum, verifyTypeEnum, receiver)) {
            return new ResultMap<>(CommonConstant.SUCCESS, VerifyTypeEnum.PHONE.code().equals(type) ?AuthConstant.SUCCESS_MSG_CODE_SEND:AuthConstant.SUCCESS_MSG_CODE_SEND_MAIL, true);
        } else {
            return new ResultMap<>(CommonConstant.ERROR, VerifyTypeEnum.PHONE.code().equals(type) ?AuthConstant.EXCEPTION_MSG_CODE_SEND_ERROR:AuthConstant.EXCEPTION_MAIL_CODE_SEND_ERROR, false);
        }
    }

    /**
     * 检查用户是否存在(用于忘记密码)
     * @param code 图片验证码
     * @param account 账户
     * @return Boolean
     */
    @PostMapping("/checkExist")
    public ResultMap<Boolean> checkExist(@NotBlank(message = "key不能为空") @RequestParam String key,@NotBlank(message = "验证码不能为空") @RequestParam String code,@RequestParam @NotBlank(message = "手机号/邮箱 不能为空")String account) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,authService.checkExist(key,account,code));
    }

    /**
     * app端登录
     * @param param 登录参数
     * @return 返回token
     */
    @PostMapping("/appGetToken")
    public AppResultMap<LoginInfoVo> appGetToken(@RequestBody AppRequestParam<RelationParam> param){
        LoginInfoVo loginInfoVo = authService.appGetToken(param.getData());
        if(StringUtils.isEmpty(loginInfoVo.getMessage())){
            return new AppResultMap<>(CommonConstant.SUCCESS,CommonConstant.CODE_SUCCESS,loginInfoVo);
        }else{
            return new AppResultMap<>(CommonConstant.EXCEPTION_CODE_RUNTIME_ERROR,loginInfoVo.getMessage(),loginInfoVo);
        }

    }

    @PostMapping("/logout")
    public AppResultMap<String> logOut(@RequestBody AppLogoutParam param) {
        authService.logout(param);
        return new AppResultMap<>(CommonConstant.SUCCESS,CommonConstant.CODE_SUCCESS,"登出成功");
    }

    /**
     * 清除设备信息并清除token信息
     * @param id 用户id
     * @return
     */
    @GetMapping("/clearDevice")
    public AppResultMap<String> clearDevice(@RequestParam("id") String id){
        return new AppResultMap<>(CommonConstant.SUCCESS, CommonConstant.CODE_SUCCESS, authService.clearDevice(id));
    }

}
