package cn.kdan.pdf.backend.core.service;

import cn.kdan.pdf.backend.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.backend.core.enums.ImageCodeTypeEnum;
import cn.kdan.pdf.backend.core.enums.VerifyTypeEnum;
import cn.kdan.pdf.backend.core.model.OauthClientDetails;
import cn.kdan.pdf.backend.core.params.AppLogoutParam;
import cn.kdan.pdf.backend.core.params.RelationParam;
import cn.kdan.pdf.backend.core.pojo.app.LoginInfoVo;
import cn.kdan.pdf.backend.core.pojo.oauth2.TokenPOJO;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * 认证service（提供客户端的获取方法以及获取授权码和token的方法）
 *
 * @author tangxiangan
 */
public interface AuthService {

    /**
     * 获取oauth2 Client配置（用于oauth的客户端凭证校验）
     *
     * @param clientId clientId
     * @return OauthClientDetails
     */
    OauthClientDetails getById(String clientId);

    /**
     * 列表获取oauth2配置（用于oauth的客户端凭证校验）
     *
     * @return List
     */
    List<OauthClientDetails> listAll();

    /**
     * 获取授权码
     *
     * @param userId    用户id
     * @param principal 凭证
     * @return String
     */
    String getAuthorizeCode(String userId, Principal principal);

    /**
     * 获取token
     *
     * @param code 授权码
     * @return TokenPOJO
     */
    TokenPOJO getToken(String code);

    /**
     * 根据用户的id直接获取token(用于第三方回调)
     *
     * @param userId 用户id
     * @return TokenPOJO
     */
    TokenPOJO getTokenByUserId(String userId);

    /**
     * 初始化验证码
     *
     * @param response  response
     * @param session   session
     * @param type      图片用途
     * @param width     图片的宽度
     * @param height    图片的长度
     * @param codeCount 验证码字符个数
     * @param lineCount 干扰线的数量
     * @throws IOException IOException
     */
    void generateImageCode(HttpServletResponse response, HttpSession session, String type, int width, int height, int codeCount, int lineCount) throws IOException;

    /**
     * 判断图片验证码
     *
     * @param key  session
     * @param type 验证码类型（登录或者忘记秘密）
     * @param code 验证码
     * @return boolean
     */
    boolean isImageCodeValid(String key, ImageCodeTypeEnum type, String code);

    /**
     * 获取验证码（发送短信，或者邮件）
     *
     * @param action   动作（例如登录）
     * @param type     类型 （发短信还是邮件）
     * @param receiver 接受人
     * @return boolean
     */
    boolean getVerifyCode(CaptchaActionEnum action, VerifyTypeEnum type, String receiver);


    /**
     * 判断账户是否存在
     *
     * @param key     session
     * @param account 手机号或者邮箱
     * @param code    验证码
     * @return boolean
     */
    boolean checkExist(String key, String account, String code);

    /**
     * 检查验证码
     *
     * @param action   动作
     * @param receiver 接收人
     * @param code     验证码
     * @return boolean
     */
    boolean checkVerifyCode(String action, String receiver, String code);

    /**
     * app端获取token登录
     * @param param 登录参数
     * @return token
     */
    LoginInfoVo appGetToken(RelationParam param);

    /**
     * 登出
     * @param param
     */
    void logout(AppLogoutParam param);

    /**
     * 失效token
     * @param token 用户token
     */
    void invalidToken(String token);

    /**
     * 清除设备信息并失效token
     * @param id 用户id
     * @return
     */
    String clearDevice(String id);
}
