package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.OrderConstant;
import cn.kdan.pdf.backend.core.enums.ClientEnum;
import cn.kdan.pdf.backend.core.model.Subscriptions;
import cn.kdan.pdf.backend.core.service.AlipayService;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayRequest;
import com.alipay.api.AlipayResponse;
import com.alipay.api.domain.AlipayTradePagePayModel;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.request.AlipayTradeCloseRequest;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.alipay.api.request.AlipayTradeQueryRequest;
import com.alipay.api.response.AlipayTradeCloseResponse;
import com.alipay.api.response.AlipayTradeQueryResponse;
import enums.CommonEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import utils.DateUtils;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;

@Slf4j
@Service
public class AlipayServiceImpl implements AlipayService {

    @Autowired
    private AlipayClient alipayClient;

    @Autowired
    private Environment config;

    /**
     * 支付宝开放平台接收 request 请求对象后
     * 会为开发者生成一个html形式的form表单,包含自动提交的脚本
     * 将form表单字符串返回给前端
     * 前端自动提交脚本
     * 表单会自动提交到 action熟悉只想的支付宝开放平台中 为用户展示一个支付页面
     *
     * @param subject      交易产品名称
     * @param tradeNo      交易号
     * @param price        交易价格
     * @param subscription 订阅记录
     * @return 支付宝调用结果
     */
    @Override
    public String createAlipayPageUrl(String subject, String tradeNo, Float price, Subscriptions subscription, String client) {
        //封装bizModel对象
        AlipayTradePagePayModel model =new AlipayTradePagePayModel();
        model.setOutTradeNo(tradeNo);
        model.setTotalAmount(price.toString());
        model.setSubject(subject);
        if(ClientEnum.WEB.value().equals(client)) {
            model.setProductCode("FAST_INSTANT_TRADE_PAY");
        }else {
            model.setProductCode("QUICK_MSECURITY_PAY");
        }

        Date date = DateUtils.addHours(new Date(), OrderConstant.ALIPAY_EXPIRE_TIME, 1);
        String timeExpire = DateUtils.dateToString(date, "yyyy-MM-dd HH:mm:ss");
        //todo 暂定当前时间12小时后过期
        model.setTimeExpire(timeExpire);

        // 最终目的是获取url路径,联调支付宝
        AlipayRequest request;
        if(ClientEnum.WEB.value().equals(client)) {
            request = new AlipayTradePagePayRequest();
        }else {
            request = new AlipayTradeAppPayRequest();
        }
//        AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();
        if(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value().equals(subscription.getTargetType())){
            request.setReturnUrl(config.getProperty("ali.pay.setPricingReturnUrl"));
        }else if(CommonEnum.SubscriptionTypeEnum.PRICING.value().equals(subscription.getTargetType())){
            request.setReturnUrl(config.getProperty("ali.pay.pricingReturnUrl"));
        }else{
            request.setReturnUrl(config.getProperty("ali.pay.returnUrl"));
        }
        //设置回调return_url、响应url
        request.setNotifyUrl(config.getProperty("ali.pay.notifyUrl"));
        request.setBizModel(model);
        AlipayResponse response = null;
        try {
            log.info("支付宝支付调用参数：{}", JSONObject.toJSONString(request));
            //get方式请求，即生成url链接
            if(ClientEnum.WEB.value().equals(client)) {
                response = alipayClient.pageExecute(request, "GET");
            }else {
                response = alipayClient.sdkExecute(request);
            }
            log.info("支付宝支付调用成功：{}", JSONObject.toJSONString(response));
        } catch (AlipayApiException e) {
            log.error("支付宝支付调用失败：{}", e.getMessage());
            throw new BackendRuntimeException("支付宝支付调用失败：" + e.getMessage());
        }
        return response.getBody();
    }

    @Override
    public String query(String tradeNo) {
        AlipayTradeQueryRequest request = new AlipayTradeQueryRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("out_trade_no", tradeNo);
        request.setBizContent(bizContent.toString());

        String result = "";
        AlipayTradeQueryResponse response = null;
        try {
            log.info("查询订单开始，request{}",JSONObject.toJSONString(request));
            response = alipayClient.execute(request);
            result = response.getBody();
            if (response.isSuccess()) {
                log.info("查询订单状态成功");
            } else {
                log.info("查询订单状态失败，{}",JSONObject.toJSONString(result));
            }
        }catch (Exception e){
            log.info("查询订单状态失败，tradeNo："+tradeNo);
            throw new BackendRuntimeException("查询订单状态失败");
        }
        return result;
    }

    @Override
    public String close(String tradeNo) throws AlipayApiException {
        AlipayTradeCloseRequest request = new AlipayTradeCloseRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("out_trade_no", tradeNo);
        request.setBizContent(bizContent.toString());

        AlipayTradeCloseResponse response = alipayClient.execute(request);
        String result = response.getBody();
        if (response.isSuccess()) {
            log.info("关闭订单成功");
        } else {
            log.info("关闭订单失败,{}",JSONObject.toJSONString(result));
        }

        return result;
    }

    char[] hexDigits = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
            'a', 'b', 'c', 'd', 'e', 'f' };

    public static void main(String[] args) throws NoSuchAlgorithmException {
        String account = "86-15111100836";
        String str = sha1(account);
        System.out.println(str);
    }

    /**
     * sha1加密
     * @param data
     * @return
     * @throws NoSuchAlgorithmException
     */
    public static String sha1(String data) throws NoSuchAlgorithmException {
/*        //加盐   更安全一些
        data += "lyz";*/
        //信息摘要器                                算法名称
        MessageDigest md = MessageDigest.getInstance("SHA1");
        //把字符串转为字节数组
        byte[] b = data.getBytes();
        //使用指定的字节来更新我们的摘要
        md.update(b);
        //获取密文  （完成摘要计算）
        byte[] b2 = md.digest();
        //获取计算的长度
        int len = b2.length;
        //16进制字符串
        String str = "0123456789abcdef";
        //把字符串转为字符串数组
        char[] ch = str.toCharArray();

        //创建一个40位长度的字节数组
        char[] chs = new char[len*2];
        //循环20次
        for(int i=0,k=0;i<len;i++) {
            //获取摘要计算后的字节数组中的每个字节
            byte b3 = b2[i];
            // >>>:无符号右移
            // &:按位与
            //0xf:0-15的数字
            chs[k++] = ch[b3 >>> 4 & 0xf];
            chs[k++] = ch[b3 & 0xf];
        }

        //字符数组转为字符串
        return new String(chs);
    }
}
