package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.constant.MembersConstant;
import cn.kdan.pdf.backend.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.backend.core.enums.ImageCodeTypeEnum;
import cn.kdan.pdf.backend.core.enums.ProviderEnum;
import cn.kdan.pdf.backend.core.enums.VerifyTypeEnum;
import cn.kdan.pdf.backend.core.mapper.OauthClientDetailsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.params.AppLoginParam;
import cn.kdan.pdf.backend.core.params.AppLogoutParam;
import cn.kdan.pdf.backend.core.params.RelationParam;
import cn.kdan.pdf.backend.core.pojo.app.LoginInfoVo;
import cn.kdan.pdf.backend.core.pojo.app.MemberVo;
import cn.kdan.pdf.backend.core.pojo.oauth2.TokenPOJO;
import cn.kdan.pdf.backend.core.pojo.third.Tencent;
import cn.kdan.pdf.backend.core.pojo.third.Wechat;
import cn.kdan.pdf.backend.core.properties.Oauth2LoginProperties;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.CommonBusinessUtils;
import cn.kdan.pdf.backend.core.utils.TencentUtils;
import cn.kdan.pdf.backend.core.utils.TokenUtils;
import cn.kdan.pdf.backend.core.utils.WechatUtils;
import constant.CommonConstant;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.http.HttpHeaders;
import org.apache.http.entity.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.provider.token.TokenStore;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import pojo.CustomUserDetails;
import pojo.Device;
import pojo.Location;
import pojo.ResultMap;
import utils.*;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.Principal;
import java.util.*;

/**
 * 认证实现类
 *
 * @author tangxiangan
 */
@Service
@Slf4j
public class AuthServiceImpl implements AuthService {


    @Value("${web.url}")
    private String url;

    @Autowired
    private OauthClientDetailsMapper oauthClientDetailsMapper;

    @Autowired
    private SMSUtils smsUtils;

    @Autowired
    private EmailUtils emailUtils;

    @Autowired
    private Oauth2LoginProperties oauth2LoginProperties;

    @Autowired
    private RedisUtils<String, String> redisUtils;

    @Value("${security.oauth2.client.client-id:}")
    private String clientId;

    @Value("${security.oauth2.client.client-secret:}")
    private String secretKey;

    @Resource
    private UserService userService;
    @Resource
    private SocialAccountService socialAccountService;
    @Resource
    private DeviceService deviceService;
    @Resource
    private LocationService locationService;
    @Resource
    private MemberServiceImpl memberService;
    @Resource
    private AvatarService avatarService;

    @Override
    public OauthClientDetails getById(String clientId) {
        return oauthClientDetailsMapper.selectByPrimaryKey(clientId);
    }

    @Override
    public List<OauthClientDetails> listAll() {
        return oauthClientDetailsMapper.selectByExample(new OauthClientDetailsExample());
    }

    @Override
    public String getAuthorizeCode(String userId, Principal principal) {
        return TokenUtils.getAuthorizeCode(clientId, principal);
    }

    @Override
    public TokenPOJO getToken(String code) {
        OAuth2AccessToken token = TokenUtils.getToken(code, clientId, secretKey);
        return TokenUtils.convert(token);
    }

    @Override
    public TokenPOJO getTokenByUserId(String userId) {
        Members members = userService.getById(userId);
        Set<SimpleGrantedAuthority> authorities = new LinkedHashSet<>();
        String account = members.getId();
        CustomUserDetails principal = new CustomUserDetails(members.getId(),
                account,
                members.getPasswordDigest(),
                true,
                true,
                true,
                true,
                authorities);
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        OauthClientDetails clientDetails = this.getUsedClient();
        if (clientDetails == null) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PLEASE_ADD_AUTH_CONFIG);
        }
        //应用内部登录的授权码
        String authCode = TokenUtils.getAuthorizeCode(clientDetails.getClientId(), authenticationToken);
        OAuth2AccessToken token = TokenUtils.getToken(authCode, clientId, secretKey);
        return TokenUtils.convert(token);
    }

    private OauthClientDetails getUsedClient() {

        OauthClientDetailsExample example = new OauthClientDetailsExample();
        example
                .createCriteria()
                .andStateEqualTo(ValidStatusEnum.VALID.value());

        List<OauthClientDetails> oauthClientDetailsList = oauthClientDetailsMapper.selectByExample(example);

        if (CollectionUtils.isEmpty(oauthClientDetailsList)) {
            return null;
        } else {
            return oauthClientDetailsList.get(0);
        }
    }


    @Override
    public void generateImageCode(HttpServletResponse response, HttpSession session, String type, int width, int height, int codeCount, int lineCount) throws IOException {
        ImageCodeTypeEnum imageCodeType;
        //设置相应类型,告诉浏览器输出的内容为图片
        response.setContentType(ContentType.IMAGE_PNG.getMimeType());
        //设置响应头信息，告诉浏览器不要缓存此内容
        response.setHeader(HttpHeaders.PRAGMA, CommonConstant.STRING_NO_CACHE);
        if (StringUtils.isEmpty(type)) {
            imageCodeType = ImageCodeTypeEnum.LOGIN;
        } else {
            imageCodeType = ImageCodeTypeEnum.getEnumByValue(type);
        }
        Map<String, Object> map = ImageCodeUtils.generate(width, height, codeCount, lineCount);
        BufferedImage image = (BufferedImage) map.get(CommonConstant.STRING_IMAGE);
        redisUtils.hset(AuthConstant.SESSION_ + session.getId(), imageCodeType.value(), (String) map.get(CommonConstant.STRING_CODE).toString().toLowerCase());
        response.setHeader(CommonConstant.AUTH_TOKEN, session.getId());
        ServletOutputStream os = response.getOutputStream();
        ImageIO.write(image, CommonConstant.STRING_PNG, os);
        os.close();
    }

    @Override
    public boolean isImageCodeValid(String key, ImageCodeTypeEnum type, String code) {
        String redisCode = redisUtils.hget(AuthConstant.SESSION_ + key, type.value().toLowerCase());
        redisUtils.hdel(AuthConstant.SESSION_ + key, type.value());
        // modify 禅道11949 图片验证码区分大小写 2022年12月2日13:27:06
        return !StringUtils.isEmpty(code) && code.equals(redisCode);
    }

    @Override
    public boolean getVerifyCode(CaptchaActionEnum action, VerifyTypeEnum type, String receiver) {
        boolean flag = false;
        if (!CaptchaActionEnum.FORGET_PASSWORD.value().equals(action.value())&&redisUtils.hexists(AuthConstant.VERIFY_CODE_TIME_OUT + action.value(), receiver)) {
            throw new BackendRuntimeException(AuthConstant.VERIFY_CODE_SEND_TOO_QUICKLY);
        }
        String code = CommonUtils.generateVerifyCode();
        switch (type) {
            //发送手机验证码
            case PHONE:
                //忘记密码时手机验证 发送手机短信
                if (smsUtils.sendSMS(Collections.singletonList(receiver))) {
                    flag = true;
                }
                break;
            //发送邮件
            case EMAIL:
                // 判断是否属于黑名单格式的邮箱
                if(!CommonBusinessUtils.verifyBlackEmail(receiver)) {
                    //忘记密码时邮箱验证 发送重置密码邮件
                    if (CaptchaActionEnum.FORGET_PASSWORD.value().equals(action.value())) {
                        String content = String.format(CommonConstant.resetPasswordEmailTemplate, url + "resetPsw?verifyCode=" + code + "&account=" + receiver);
                        flag = emailUtils.sendMail(action.subject(), content, Collections.singletonList(receiver));
                    } else {
                        String content = String.format(CommonConstant.registerEmailTemplate, code);
                        flag = emailUtils.sendMail(action.subject(), content, Collections.singletonList(receiver));
                    }
                }
                break;
            default:
                break;
        }
        //存入缓存
        if (flag) {
            if (CaptchaActionEnum.FORGET_PASSWORD.value().equals(action.value())) {
                // 注册用户：verifyCode_1：{13433334444,code}
                redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + action.value(), receiver, code, AuthConstant.VERIFY_CODE_KEY_EXPIRE_TIME);
            }else{
                // 注册用户：verifyCode_0：{test@qq.com,code}
                redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + action.value(), receiver, code, AuthConstant.EMAIL_VERIFY_CODE_KEY_EXPIRE_TIME);
            }
            //再次发送计时
            redisUtils.hset(AuthConstant.VERIFY_CODE_TIME_OUT + action.value(), receiver, "1", AuthConstant.VERIFY_CODE_KEY_RESEND_TIME);
        }
        return flag;
    }

    @Override
    public boolean checkExist(String key, String account, String code) {
        if (!isImageCodeValid(key, ImageCodeTypeEnum.FORGET_PASSWORD, code)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_IMAGE_CODE_ERROR);
        }
        if (ObjectUtils.isEmpty(userService.getByAccount(account))) {
            throw new BackendRuntimeException(MembersConstant.EXCEPTION_MSG_USER_NOT_FOUND);
        }
        return Boolean.TRUE;
    }

    @Override
    public boolean checkVerifyCode(String action, String receiver, String code) {
        String captchaCode = redisUtils.hget(AuthConstant.VERIFY_CODE_KEY + action, receiver);
        return StringUtils.isNotEmpty(code) && code.equals(captchaCode);
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public LoginInfoVo appGetToken(RelationParam param) {
        LoginInfoVo vo = new LoginInfoVo();
        AppLoginParam attribute= param.getAttribute();
        String provider = attribute.getProvider();
        String password = attribute.getPassword();
        if(StringUtils.isBlank(password)){
            password = "false";
        }
        String openid = attribute.getOpenid();
        String accessToken = attribute.getAccessToken();
        String memberId = "";
        MemberVo memberVo = new MemberVo();
        memberVo.setStatus("verified");
        memberVo.setTimeZone("UTC");
        memberVo.setPhoneZone("86");

        //provider不为空则为QQ、微信登录（tencent、wechat）
        if(StringUtils.isNotEmpty(provider)){
            // 判断是否存在用户数据，存在则不新增members数据
            if(ProviderEnum.TENCENT.getName().equals(provider)){
                Tencent tencent = TencentUtils.getUserInfo(accessToken);
                memberVo.setAvatarUrl(tencent.getFigureUrl());
                memberVo.setName(tencent.getNickname());
                memberId = insertMemberAndSocialAccount(provider, password, tencent, null);
            }else if(ProviderEnum.WECHAT.getName().equals(provider)){
                Wechat wechat = WechatUtils.getUserInfo(openid, accessToken);
                memberVo.setName(wechat.getNickname());
                memberVo.setAvatarUrl(wechat.getHeadImgUrl());
                memberId = insertMemberAndSocialAccount(provider, password,null,wechat);
            }
        }else{
            //手机号、邮箱登录
            String account = attribute.getAccount();
            if(StringUtils.isEmpty(account)){
                throw new BackendRuntimeException("请输入账号");
            }
            //根据账号查询members数据
            Members member = userService.getByAccount(attribute.getAccount());
            String mobileRegex = CommonConstant.mobileRegex;
            //如果members为空并且account为手机号，新增一条members数据；邮箱不支持注册，只允许输入账号密码登录
            if(account.matches(mobileRegex)){
                if(ObjectUtils.isEmpty(member)) {
                    member = new Members();
                    member.setName(String.valueOf(System.currentTimeMillis()).substring(0, 10));
                    member.setPhone(account);
                    memberId = memberService.insertMembers(member, password);
                    memberVo.setName(member.getName());
                    memberVo.setEmail(member.getEmail());
                    memberVo.setPhone(member.getPhone());
                    memberVo.setPhoneZone(member.getPhoneZone());
                    memberVo.setTimeZone(member.getTimeZone());
                    memberVo.setAvatarUrl(avatarService.selectAvatarByMemberId(memberId));
                }else {
                    if("false".equals(password)){
                        memberVo.setName(member.getName());
                        memberVo.setEmail(member.getEmail());
                        memberVo.setPhone(member.getPhone());
                        memberVo.setPhoneZone(member.getPhoneZone());
                        memberVo.setTimeZone(member.getTimeZone());
                        memberId = member.getId();
                        memberVo.setAvatarUrl(avatarService.selectAvatarByMemberId(memberId));
                    }
                }
            }else {
                //如果用户存在将用户id返回，否则其他情况(邮箱登录)全部抛异常：用户未注册
                if (ObjectUtils.isNotEmpty(member)) {
                    String passwordDigest = member.getPasswordDigest();
                    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
                    if(encoder.matches(password, passwordDigest)){
                        memberVo.setName(member.getName());
                        memberVo.setEmail(member.getEmail());
                        memberVo.setPhone(member.getPhone());
                        memberVo.setPhoneZone(member.getPhoneZone());
                        memberVo.setTimeZone(member.getTimeZone());
                        memberId = member.getId();
                        memberVo.setAvatarUrl(avatarService.selectAvatarByMemberId(memberId));
                    }else {
                        throw new BackendRuntimeException("账号/密码输入有误");
                    }
                } else {
                    throw new UsernameNotFoundException(JsonUtils.getJsonString(new ResultMap<>(AuthConstant.EXCEPTION_CODE_USER_NOT_FOUND, AuthConstant.EXCEPTION_MSG_USER_NOT_FOUND)));
                }
            }
        }

        //增加设备数据
        Device loginDevice = param.getDevice();
        Devices queryDevice;
        try {
            queryDevice = deviceService.selectByMemberId(memberId, loginDevice.getUuid());
        }catch (BackendRuntimeException e){
            vo.setId(memberId);
            vo.setMessage(e.getMessage());
            return vo;
        }
        // 如果查询的设备为空，并且传参进来的device不为空，新增一条device数据
        if(ObjectUtils.isEmpty(queryDevice)) {
            if (StringUtils.isNotBlank(loginDevice.getUuid())) {
                Devices device = new Devices();
                BeanUtils.copyProperties(loginDevice, device);
                device.setId(CommonUtils.generateId());
                device.setMemberId(memberId);
                memberVo.setTimeZone(loginDevice.getTimeZone());
                deviceService.insert(device);
            } else {
                memberVo.setTimeZone("UTC");
            }
        }else{
            // 根据uuid和memberId修改device设备信息
            String uuid = queryDevice.getUuid();
            if (StringUtils.isNotBlank(uuid)) {
                BeanUtils.copyProperties(loginDevice, queryDevice);
                deviceService.update(queryDevice, uuid);
            }
        }

        //查询是否存在位置信息，存在则更新，不存在则新增
        Location loginLocation = param.getLocation();
        Locations queryLocation = locationService.selectByMemberId(memberId);
        if (ObjectUtils.isEmpty(queryLocation)) {
            if(StringUtils.isNotEmpty(loginLocation.getCountry())) {
                Locations location = new Locations();
                BeanUtils.copyProperties(loginLocation, location);
                location.setId(CommonUtils.generateId());
                location.setMemberId(memberId);
                locationService.insert(location);
            }
        }else{
            if(StringUtils.isNotEmpty(loginLocation.getCountry())) {
                BeanUtils.copyProperties(loginLocation, queryLocation);
                locationService.update(queryLocation);
            }
        }
        //根据用户id查询token
        TokenPOJO tokenPOJO = getTokenByUserId(memberId);
        // 更新accessToken信息至members表
        Members members = memberService.getById(memberId);
        members.setActivationToken(tokenPOJO.getAccessToken());
        members.setResetToken(tokenPOJO.getRefreshToken());
        members.setUpdatedAt(new Date());
        memberService.update(members);
        vo.setToken(tokenPOJO);
        vo.setId(memberId);
        //获取用户信息
        memberVo.setId(memberId);
        vo.setMember(memberVo);
        return vo;
    }

    /**
     * 判断members表是否存在数据，不存在则插入用户和社交账户表，并返回memberId
     * 存在则不插入直接返回memberId
     *
     * @param provider 第三方平台
     * @param tencent qq用户信息
     * @param wechat 微信用户信息
     * @return 用户id
     */
    private String insertMemberAndSocialAccount(String provider,String password, Tencent tencent, Wechat wechat) {
        String memberId = "";
        if(ProviderEnum.TENCENT.getName().equals(provider)){
            SocialAccounts socialAccounts = socialAccountService.selectByAccountId(tencent.getUnionId());
            if(ObjectUtils.isEmpty(socialAccounts)){
                Members member = new Members();
                setMembersName(tencent.getNickname(), member);
                String id = memberService.insertMembers(member, password);
                insertSocialAccount(provider, id, tencent.getUnionId(), tencent.getFigureUrl(), tencent.getNickname());
                memberId = id;
            } else{
                memberId = socialAccounts.getMemberId();
            }
        }else if(ProviderEnum.WECHAT.getName().equals(provider)){
            SocialAccounts socialAccounts = socialAccountService.selectByAccountId(wechat.getUnionId());
            if(ObjectUtils.isEmpty(socialAccounts)){
                Members member = new Members();
                setMembersName(wechat.getNickname(),member);
                String id = memberService.insertMembers(member, password);
                insertSocialAccount(provider, id, wechat.getUnionId(), wechat.getHeadImgUrl(), wechat.getNickname());
                memberId = id;
            } else{
                memberId = socialAccounts.getMemberId();
            }
        }
        return memberId;
    }

    /**
     * 设置用户名（用户名唯一性）
     * @param name
     * @param member
     */
    private void setMembersName(String name, Members member) {
        Members queryMembers = memberService.selectByName(name);
        if(queryMembers != null){
            //todo 待考虑第三方名称与数据库数据重名后该怎么赋值
            member.setName(String.valueOf(System.currentTimeMillis()).substring(0, 10));
        }else{
            member.setName(name);
        }
    }

    /**
     * 新增社交账号表数据
     * @param provider tencent/wechat
     * @param memberId 用户id
     * @param unionId accountId（第三方返回的唯一标识）
     * @param figureUrl 头像地址
     * @param nickname 昵称
     */
    private void insertSocialAccount(String provider, String memberId, String unionId, String figureUrl, String nickname) {
        SocialAccounts account = new SocialAccounts();
        account.setId(CommonUtils.generateId());
        account.setMemberId(memberId);
        account.setAccountId(unionId);
        account.setAvatar(figureUrl);
        account.setName(nickname);
        Integer providerValue = ProviderEnum.getCodeByName(provider);
        account.setProvider(providerValue);
        socialAccountService.insertSocialAccount(account);
    }

    @Resource
    @Lazy
    private TokenStore tokenStore;

    @Override
    public void logout(AppLogoutParam param) {
        String accessToken = param.getAccessToken();
        Assert.notNull(accessToken, "传参错误，入参accessToken为空");
        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(accessToken);
        Device device = param.getDevice();
        if (oAuth2AccessToken != null) {
            deviceService.deleteByUuid(device.getUuid(),param.getMemberId());
            OAuth2RefreshToken oAuth2RefreshToken = oAuth2AccessToken.getRefreshToken();
            //从tokenStore中移除token
            tokenStore.removeAccessToken(oAuth2AccessToken);
            tokenStore.removeRefreshToken(oAuth2RefreshToken);
            tokenStore.removeAccessTokenUsingRefreshToken(oAuth2RefreshToken);
        } else {
            throw new BackendRuntimeException("token已失效，请勿重复登出");
        }
    }

    @Override
    public void invalidToken(String token) {
        OAuth2AccessToken oAuth2AccessToken = tokenStore.readAccessToken(token);
        if (oAuth2AccessToken != null) {
            OAuth2RefreshToken oAuth2RefreshToken = oAuth2AccessToken.getRefreshToken();
            //从tokenStore中移除token
            tokenStore.removeAccessToken(oAuth2AccessToken);
            tokenStore.removeRefreshToken(oAuth2RefreshToken);
            tokenStore.removeAccessTokenUsingRefreshToken(oAuth2RefreshToken);
        }
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public String clearDevice(String id) {
        Members member = memberService.getById(id);
        //清除设备信息
        deviceService.clearDevice(id);
        invalidToken(member.getActivationToken());

        // 置空用户token信息
        member.setUpdatedAt(new Date());
        member.setActivationToken("");
        member.setResetToken("");
        memberService.update(member);
        return "清除设备信息成功";
    }
}
