package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.AuthConstant;
import cn.kdan.pdf.backend.core.constant.MembersConstant;
import cn.kdan.pdf.backend.core.enums.*;
import cn.kdan.pdf.backend.core.mapper.*;
import cn.kdan.pdf.backend.core.mapper.ext.ExtMembersMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.model.member.*;
import cn.kdan.pdf.backend.core.params.BindAccountParam;
import cn.kdan.pdf.backend.core.params.CheckExistsParam;
import cn.kdan.pdf.backend.core.params.UserResetPwdParams;
import cn.kdan.pdf.backend.core.pojo.ExistsMemberVo;
import cn.kdan.pdf.backend.core.pojo.app.BindAccountVo;
import cn.kdan.pdf.backend.core.pojo.app.MemberInfoVo;
import cn.kdan.pdf.backend.core.pojo.app.MemberVo;
import cn.kdan.pdf.backend.core.pojo.app.SocialAccountVo;
import cn.kdan.pdf.backend.core.service.*;
import constant.CommonConstant;
import enums.CommonEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import pojo.CustomUserDetails;
import utils.CommonUtils;
import utils.RedisUtils;
import utils.SMSUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Service
public class MemberServiceImpl implements MemberService {

    @Autowired
    private MembersMapper membersMapper;
    @Autowired
    private AuthService authService;
    @Autowired
    private SubscriptionsMapper subscriptionsMapper;
    @Autowired
    private PricingsMapper pricingsMapper;
    @Autowired
    private SetPricingsMapper setPricingsMapper;
    @Autowired
    private SMSUtils smsUtils;
    @Autowired
    private AvatarService avatarService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private ExtMembersMapper extMembersMapper;
    @Autowired
    private SocialAccountService socialAccountService;
    @Autowired
    private UserService userService;

    /**
     * 默认时区
     */
    private final static String DEFAULT_TIME_ZONE = "UTC";
    /**
     * 手机服务区
     */
    private final static String DEFAULT_PHONE_ZONE = "86";
    /**
     * 默认空间
     */
    private final static Integer DEFAULT_SPACE = 500;
    /**
     * 初始券数
     */
    private final static Integer INIT_POINTS = 10;
    /**
     * 默认用户浏览次数
     */
    private final static Integer DEFAULT_VIEW_TIMES = 0;
    /**
     * 匿名用户
     */
    private final static String ANONYMOUS_USER = "anonymousUser";


    /**
     * 用户注册接口：手机号需要先验证验证码是否存在以及正确性
     *
     * @param req 用户注册所填
     * @return 返回注册提示信息
     */
    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public MemberRegisterResp register(MemberRegisterReq req) {
        String username = req.getUsername();
        MemberRegisterResp resp = new MemberRegisterResp();

        // 密码校验
        String password = passwordValidate(req.getPassword(), req.getPasswordConfirm());
        // 验证用户名是否符合手机号或邮箱的格式
        String mobileRegex = CommonConstant.mobileRegex;
        String emailRegex = CommonConstant.emailRegex;
        if (!username.matches(mobileRegex) && !username.matches(emailRegex)) {
            resp.setMessage("手机号/邮箱格式不对");
            throw new BackendRuntimeException("手机号/邮箱格式不对");
        }
        // 手机号需要先验证验证码是否存在以及正确性
        if (username.matches(mobileRegex)) {
            // 验证手机验证码
            if (!smsUtils.verifySMS(Collections.singletonList(username), req.getVerifyCode())) {
                throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_CODE_ERROR);
            } else {
                // 验证成功删除验证码缓存
                redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.USER_REGISTER.value(), username);
            }
        }else{
            // 验证邮箱验证码
            String verifyCode = redisUtils.hget(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.USER_REGISTER.value(), username);
            if (!req.getVerifyCode().equals(verifyCode)) {
                throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_CODE_ERROR);
            } else {
                // 验证成功删除验证码缓存
                redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.USER_REGISTER.value(), username);
            }
        }

        // 根据用户名查询数据库是否已有数据，判断该账号是否已注册过
        List<Members> memberList = extMembersMapper.getMemberByAccount(username);
        if (!CollectionUtils.isEmpty(memberList)) {
            resp.setMessage("该账号已被注册");
            throw new BackendRuntimeException("该账号已被注册");
        }

        // 添加数据
        Members member = new Members();
        if (username.matches(mobileRegex)) {
            member.setPhone(username);
        } else if (username.matches(emailRegex)) {
            member.setEmail(username);
        }
        //用户命名使用时间戳
        member.setName(String.valueOf(System.currentTimeMillis()).substring(0, 10));
        insertMembers(member, password);
        resp.setMessage("注册成功");
        return resp;
    }

    /**
     * 为members设置基础值并插入数据库
     * @param member member数据
     * @param password 密码
     * @return  用户id
     */
    public String insertMembers(Members member, String password) {
        String id = CommonUtils.generateId();
        member.setId(id);
        Date date = new Date();
        //使用Bcrypt加密方式对密码进行加密处理
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        member.setPasswordDigest(encoder.encode(password));
        member.setCreatedAt(date);
        member.setUpdatedAt(date);
        member.setTimeZone(DEFAULT_TIME_ZONE);
        member.setViewedTimes(DEFAULT_VIEW_TIMES);
        member.setStatus(CommonEnum.YesOrNoEnum.NO.value());
        member.setPhoneZone(DEFAULT_PHONE_ZONE);
        member.setSubscriberType(CommonEnum.YesOrNoEnum.NO.value());
        // todo 用户账号account = sha1(86-phone / email)
        member.setAccount("");
        member.setPoints(INIT_POINTS);
        member.setTotalPoints(INIT_POINTS);
        member.setTotalSpace(DEFAULT_SPACE);
        member.setLeftSpace(new BigDecimal(DEFAULT_SPACE));
        membersMapper.insert(member);
        return id;
    }

    /**
     * 密码校验
     *
     * @param password 密码
     * @param passwordConfirm 确认密码
     * @return 密码
     */
    private String passwordValidate(String password, String passwordConfirm) {
        //密码规则校验
        if(password.contains(" ")){
            throw new BackendRuntimeException("密码不能输入空格符号");
        }
        if (!password.equals(passwordConfirm)) {
            throw new BackendRuntimeException("两次输入的密码不一致");
        }
        if (password.length() < 6) {
            throw new BackendRuntimeException("密码位数不能少于6位");
        }
        if (password.length() > 16) {
            throw new BackendRuntimeException("密码位数不能大于16位");
        }
        return password;
    }

    /**
     * 修改密码
     *
     * @param req
     */
    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void modifyPassword(ModifyPasswordReq req) {
        String newPassword = passwordValidate(req.getNewPassword(), req.getNewPasswordConfirm());
        /*
            1.根据token解析出用户名
            2.再根据用户名查询出用户的密码
            3.比对用户密码与输入的密码是否一致，一致则允许修改，不一致则不允许，返回报错原因
            4.密码一致，修改用户密码，返回提示修改成功
         */
        Members members = getCurrentUser();

        // 比对用户密码与输入的密码是否一致，一致则允许修改，不一致则不允许，返回报错原因
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        boolean flag = encoder.matches(req.getPassword(), members.getPasswordDigest());
        if (flag) {
            members.setPasswordDigest(encoder.encode(newPassword));
        } else {
            throw new BackendRuntimeException("原始密码输入错误");
        }
        membersMapper.updateByPrimaryKey(members);

    }

    /**
     * 修改昵称
     *
     * @param name 用户修改昵称
     */
    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void modifyNickname(String name) {
        if(StringUtils.isEmpty(name)){
            throw new BackendRuntimeException("用户名不能为空");
        }
        if(name.length() > 12){
            throw new BackendRuntimeException("用户名称不能超过12位");
        }
        //根据输入的用户名查询数据库是否已存在对应的用户名，有则不能修改，返回提示信息，没有则可以修改
        Members members = getCurrentUser();
        if (members.getName().equals(name)) {
            return;
        }
        MembersExample example = new MembersExample();
        MembersExample.Criteria criteria = example.createCriteria();
        criteria.andNameEqualTo(name);
        criteria.andIdNotEqualTo(members.getId());
        List<Members> list = membersMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(list)) {
            members.setName(name);
            membersMapper.updateByPrimaryKey(members);
        } else {
            throw new BackendRuntimeException("用户名已存在");
        }
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @Override
    @Transactional
    public MemberInfoResp getMemberDetail(String id) {
        Members member = null;
        if(StringUtils.isEmpty(id)) {
            member = getCurrentUser();
        }else{
            member = membersMapper.selectByPrimaryKey(id);
        }

        //封装用户信息
        MemberInfo memberInfo = new MemberInfo();
        BeanUtils.copyProperties(member, memberInfo);
        if (member.getUsedSpace() != null) {
            BigDecimal usedSpacePercent = member.getUsedSpace().divide(new BigDecimal(member.getTotalSpace()), 2);
            memberInfo.setUsedSpacePercent(usedSpacePercent);
        } else {
            memberInfo.setUsedSpacePercent(BigDecimal.ZERO);
        }

        memberInfo.setAvatarUrl(avatarService.selectAvatarByMemberId(member.getId()));

        //根据用户id查询订阅数据
        SubscriptionsExample example = new SubscriptionsExample();
        SubscriptionsExample.Criteria criteria = example.createCriteria();
        criteria.andMemberIdEqualTo(memberInfo.getId());
        criteria.andStatusNotBetween(0, 1);
        example.setOrderByClause("updated_at DESC");
        List<Subscriptions> subscriptionsList = subscriptionsMapper.selectByExample(example);

        //根据targetType区分券服务与会员服务数据
        List<SubPricing> subPricingList = new ArrayList<>();
        List<SubSetPricing> subSetPricingList = new ArrayList<>();
        Map<Integer, Pricings> pricingMap = getPricingMap();
        Map<Integer, SetPricings> setPricingMap = getSetPricingMap();
        for (Subscriptions subscription : subscriptionsList) {
            if (subscription.getTargetType().equals(CommonEnum.SubscriptionTypeEnum.PRICING.value())) {
                SubPricing subPricing = new SubPricing();
                BeanUtils.copyProperties(subscription, subPricing);
                subPricing.setPricings(pricingMap.get(subscription.getTargetId()));
                subPricing.setStartDate(subscription.getCreatedAt());
                subPricingList.add(subPricing);
            } else if (subscription.getTargetType().equals(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value())) {
                // 不把targetId为1的显示在界面
                if(1 != subscription.getTargetId()) {
                    SubSetPricing subSetPricing = new SubSetPricing();
                    BeanUtils.copyProperties(subscription, subSetPricing);
                    subSetPricing.setSetPricings(setPricingMap.get(subscription.getTargetId()));
                    subSetPricing.setCreateDate(subscription.getCreatedAt());
                    subSetPricingList.add(subSetPricing);
                }
            }
        }
        //获取会员剩余天数
        long day = getLeftDay(subSetPricingList);
        memberInfo.setDay(day);
        //如果会员天数小于0，修改会员状态为否
        if(day == 0){
            if(SubscriberTypeEnum.yes.value().equals(member.getSubscriberType())) {
                member.setSubscriberType(SubscriberTypeEnum.no.value());
                membersMapper.updateByPrimaryKey(member);
            }
        } else {
            if(SubscriberTypeEnum.no.value().equals(member.getSubscriberType())) {
                member.setSubscriberType(SubscriberTypeEnum.yes.value());
                membersMapper.updateByPrimaryKey(member);
            }
        }
        MemberInfoResp resp = new MemberInfoResp();
        resp.setMemberInfo(memberInfo);
        resp.setPricingList(subPricingList);
        resp.setSetPricingList(subSetPricingList);
        resp.setType("members");
        return resp;
    }

    /**
     * 获取全部的券服务数据存入map中，id为key，pricing为value
     */
    public Map<Integer, Pricings> getPricingMap() {
        Map<Integer, Pricings> map = new HashMap<>();
        PricingsExample example = new PricingsExample();
        List<Pricings> pricings = pricingsMapper.selectByExample(example);
        for (Pricings pricing : pricings) {
            map.put(pricing.getId(), pricing);
        }
        return map;
    }

    /**
     * 获取全部的会员服务数据存入map中，id为key，setPricing为value
     */
    public Map<Integer, SetPricings> getSetPricingMap() {
        Map<Integer, SetPricings> map = new HashMap<>();
        SetPricingsExample example = new SetPricingsExample();
        List<SetPricings> setPricings = setPricingsMapper.selectByExample(example);
        for (SetPricings setPricing : setPricings) {
            map.put(setPricing.getId(), setPricing);
        }
        return map;
    }

    /**
     * 获取认证信息
     *
     * @return 认证信息
     */
    public static Authentication getAuthentication() {
        return SecurityContextHolder.getContext().getAuthentication();
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @Override
    public Members getCurrentUser() {
        Authentication authentication = getAuthentication();
        Object principal = authentication.getPrincipal();
        try {
            String userId = ((CustomUserDetails) (principal)).getId();
            return membersMapper.selectByPrimaryKey(userId);
        }catch (Exception e){
            log.info("getCurrentUser获取用户失败：{}",e.getMessage());
            return null;
        }
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public Integer judgeSubscribeType(String memberId) {
        SubscriptionsExample example = new SubscriptionsExample();
        SubscriptionsExample.Criteria criteria = example.createCriteria();
        criteria.andMemberIdEqualTo(memberId);
        criteria.andStatusNotBetween(0, 1);
        criteria.andTargetTypeEqualTo(GoodsTypeEnum.SetPricing.value());
        example.setOrderByClause("updated_at DESC");
        List<Subscriptions> subscriptionsList = subscriptionsMapper.selectByExample(example);
        List<SubSetPricing> subSetPricingList = new ArrayList<>();
        for (Subscriptions subscription : subscriptionsList) {
            if (subscription.getTargetType().equals(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value())) {
                SubSetPricing subSetPricing = new SubSetPricing();
                BeanUtils.copyProperties(subscription, subSetPricing);
                subSetPricingList.add(subSetPricing);
            }
        }
        //获取会员剩余天数
        long day = getLeftDay(subSetPricingList);
        //如果会员天数小于0，修改会员状态为否
        Members member = getCurrentUser();
        if(day == 0){
            if(SubscriberTypeEnum.yes.value().equals(member.getSubscriberType())) {
                member.setUpdatedAt(new Date());
                member.setSubscriberType(SubscriberTypeEnum.no.value());
                membersMapper.updateByPrimaryKey(member);
            }
            return SubscriberTypeEnum.no.value();
        }else{
            if(SubscriberTypeEnum.no.value().equals(member.getSubscriberType())) {
                member.setUpdatedAt(new Date());
                member.setSubscriberType(SubscriberTypeEnum.yes.value());
                membersMapper.updateByPrimaryKey(member);
            }
            return SubscriberTypeEnum.yes.value();
        }
    }

    @Override
    public Members getById(String userId) {
        Members members = membersMapper.selectByPrimaryKey(userId);
        if (ObjectUtils.isEmpty(members)) {
            throw new BackendRuntimeException(MembersConstant.MEMBERS_NOT_EXIST);
        }
        return members;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public boolean resetPassword(UserResetPwdParams params) {
        MembersExample example = new MembersExample();
        //如果是手机方式
        if (VerifyTypeEnum.PHONE.code().equals(params.getType())) {
            //验证手机验证码
            if (!smsUtils.verifySMS(Collections.singletonList(params.getAccount()), params.getVerifyCode())) {
                throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_CODE_ERROR);
            }
            ;
            example.createCriteria().andPhoneEqualTo(params.getAccount());
        } else {
            //根据code和邮箱验证用户
            if (!authService.checkVerifyCode(CaptchaActionEnum.FORGET_PASSWORD.value(), params.getAccount(), params.getVerifyCode())) {
                throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_RESET_PASSWORD_ERROR);
            }else {
                // 验证成功删除验证码缓存
                redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + CaptchaActionEnum.FORGET_PASSWORD.value(), params.getAccount());
            }
            example.createCriteria().andEmailEqualTo(params.getAccount());
        }
        //判断用户是否存在
        //重置密码
        List<Members> membersList = membersMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(membersList)) {
            throw new BackendRuntimeException(MembersConstant.MEMBERS_NOT_EXIST);
        }
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        String pwd = encoder.encode(params.getNewPassword());
        membersList.get(0).setPasswordDigest(pwd);
        membersMapper.updateByPrimaryKey(membersList.get(0));
        return Boolean.TRUE;
    }

    @Override
    public boolean appResetPassword(UserResetPwdParams param) {
        return authService.getVerifyCode(CaptchaActionEnum.FORGET_PASSWORD, VerifyTypeEnum.EMAIL, param.getAccount());
    }

    /**
     * 获取会员剩余天数
     */
    public long getLeftDay(List<SubSetPricing> subSetPricingList) {
        long day = 0;

        Date now = new Date();
        BigDecimal normal = new BigDecimal(86400000);
        for (SubSetPricing setPricing : subSetPricingList) {
            if(1 != setPricing.getTargetId()) {
                Date startDate = setPricing.getStartDate();
                Date endDate = setPricing.getEndDate();
                if (endDate.getTime() - now.getTime() > 0) {
                    if (startDate.getTime() - now.getTime() < 0) {
                        long l = endDate.getTime() - now.getTime();
                        BigDecimal b1 = new BigDecimal(l);
                        day += b1.divide(normal,0,RoundingMode.UP).longValue();
                    } else {
                        long l = endDate.getTime() - startDate.getTime();
                        BigDecimal b1 = new BigDecimal(l);
                        day += b1.divide(normal,0,RoundingMode.UP).longValue();
                    }
                }
            }
        }
        return day;
    }

    @Override
    @Transactional
    public void update(Members member) {
        membersMapper.updateByPrimaryKeySelective(member);
    }

    @Override
    public Members selectByName(String name) {
        MembersExample example = new MembersExample();
        example.createCriteria().andNameEqualTo(name);
        List<Members> list = membersMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public String cancelAccount() {
        Members currentUser = getCurrentUser();
        String memberId = currentUser.getId();
        //删除社交账号socialAccount表+用户members表数据
        //判断用户是否是空数据类型（未充值过券和会员服务）
        if(!StringUtils.isEmpty(memberId) && currentUser.getTotalPoints() == 10
                && SubscriberTypeEnum.no.value().equals(currentUser.getSubscriberType())){
            socialAccountService.deleteByMemberId(memberId);
            membersMapper.deleteByPrimaryKey(memberId);
        }else{
            throw new BackendRuntimeException("不允许注销该账号，因为充过钱了！");
        }
        return memberId;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public BindAccountVo bindAccount(BindAccountParam param) {
        BindAccountVo bindAccountVo = new BindAccountVo();
        List<SocialAccountVo> socialAccounts = new ArrayList<>();
        MemberVo memberVo = new MemberVo();
        Members currentUser = getCurrentUser();
        BeanUtils.copyProperties(currentUser,memberVo);
        memberVo.setStatus("verified");
        bindAccountVo.setMember(memberVo);

        String provider = param.getProvider();
        if("phone".equals(provider)){
            bindPhone(param, socialAccounts, currentUser);
            memberVo.setPhone(param.getPhone());
        } else{
            //绑定第三方账号
            socialAccounts = socialAccountService.bindSocialAccount(currentUser, param, socialAccounts);
        }
        bindAccountVo.setSocialAccounts(socialAccounts);
        return bindAccountVo;
    }

    /**
     * 手机号绑定
     * @param param 入参
     * @param socialAccounts 社交账号
     * @param currentUser 当前用户
     */
    private void bindPhone(BindAccountParam param, List<SocialAccountVo> socialAccounts, Members currentUser) {
        String phone = param.getPhone();
        if(StringUtils.isEmpty(phone)){
            throw new BackendRuntimeException("手机号不能为空");
        }
        Members member = userService.getByAccount(phone);
        if(member != null){
            //不允许绑定自己
            if(member.getId().equals(currentUser.getId())) {
                throw new BackendRuntimeException("不允许绑定自己");
            }else{
                if(member.getTotalPoints() <= 10 && SubscriberTypeEnum.no.value().equals(member.getSubscriberType())){
                    // 当该手机用户已经绑定了第三方账号的时候，不允许此次绑定
                    List<SocialAccounts> list = socialAccountService.selectByMemberId(member.getId());
                    if(!CollectionUtils.isEmpty(list)){
                        throw new BackendRuntimeException("该账号已被绑定，不允许再次绑定");
                    }
                    //删除该手机的账号数据，然后更新手机到当前用户
                    membersMapper.deleteByPrimaryKey(member.getId());
                    //更新当前账号的手机号为phone参数
                    if(!StringUtils.isEmpty(param.getPhoneZone())){
                        currentUser.setPhoneZone(param.getPhoneZone());
                    }
                    currentUser.setPhone(phone);
                    currentUser.setUpdatedAt(new Date());
                    update(currentUser);
                }else{
                    throw new BackendRuntimeException("不允许绑定该账号，因为充过钱了！");
                }
            }
        }else{
            //直接更新当前账号的手机号为phone参数
            if(!StringUtils.isEmpty(param.getPhoneZone())){
                currentUser.setPhoneZone(param.getPhoneZone());
            }
            currentUser.setPhone(phone);
            currentUser.setUpdatedAt(new Date());
            update(currentUser);
        }

        List<SocialAccounts> list = socialAccountService.selectByMemberId(currentUser.getId());
        for (SocialAccounts account : list) {
            SocialAccountVo vo = new SocialAccountVo();
            vo.setProvider(ProviderEnum.getNameByCode(account.getProvider()));
            vo.setName(account.getName());
            vo.setAccountId(account.getAccountId());
            socialAccounts.add(vo);
        }
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public BindAccountVo unbindAccount(BindAccountParam param) {
        BindAccountVo bindAccountVo = new BindAccountVo();
        List<SocialAccountVo> socialAccounts = new ArrayList<>();
        MemberVo memberVo = new MemberVo();
        Members currentUser = getCurrentUser();
        BeanUtils.copyProperties(currentUser,memberVo);
        memberVo.setStatus("verified");
        bindAccountVo.setMember(memberVo);

        String provider = param.getProvider();
        if("phone".equals(provider)){
            currentUser.setPhone("");
            bindAccountVo.getMember().setPhone("");
            update(currentUser);
            List<SocialAccounts> list = socialAccountService.selectByMemberId(currentUser.getId());
            for (SocialAccounts account : list) {
                SocialAccountVo vo = new SocialAccountVo();
                vo.setProvider(ProviderEnum.getNameByCode(account.getProvider()));
                vo.setName(account.getName());
                vo.setAccountId(account.getAccountId());
                socialAccounts.add(vo);
            }
        } else {
            socialAccountService.unbindSocialAccount(currentUser, provider, socialAccounts);
        }
        bindAccountVo.setSocialAccounts(socialAccounts);
        return bindAccountVo;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public MemberVo appModifyName(String name) {
        modifyNickname(name);
        Members currentUser = getCurrentUser();
        return createMemberVo(currentUser);
    }

    /**
     * 返回一个memberVo对象
     * @param member 当前用户
     * @return memberVo
     */
    public MemberVo createMemberVo(Members member){
        MemberVo vo = new MemberVo();
        BeanUtils.copyProperties(member,vo);
        vo.setAvatarUrl(avatarService.selectAvatarByMemberId(member.getId()));
        return vo;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public void deleteByMemberId(String memberId) {
        membersMapper.deleteByPrimaryKey(memberId);
    }

    @Override
    public MemberInfoVo appGetMemberInfo(String subscription, String withSocial) {
        MemberInfoVo memberInfoVo = new MemberInfoVo();
        MemberInfoResp memberDetail = getMemberDetail(null);
        memberInfoVo.setType("members");
        memberInfoVo.setMemberInfo(memberDetail.getMemberInfo());
        //1:显示订阅信息 0:不显示订阅信息
        if("1".equals(subscription)){
            memberInfoVo.setSetPricingList(memberDetail.getSetPricingList());
            memberInfoVo.setPricingList(memberDetail.getPricingList());
        }
        // 如果是QQ、微信登录，传入字段可实现第三方登录的信息。0:不显示， 1:显示
        if("1".equals(withSocial)){
            List<SocialAccountVo> list = new ArrayList<>();
            List<SocialAccounts> socialAccounts = socialAccountService.selectByMemberId(memberDetail.getMemberInfo().getId());
            if(!CollectionUtils.isEmpty(socialAccounts)) {
                for (SocialAccounts account : socialAccounts) {
                    SocialAccountVo vo = new SocialAccountVo();
                    BeanUtils.copyProperties(account, vo);
                    vo.setProvider(ProviderEnum.getNameByCode(account.getProvider()));
                    list.add(vo);
                }
            }
            memberInfoVo.setSocialAccountList(list);
        }
        return memberInfoVo;
    }

    @Override
    public ExistsMemberVo checkExists(CheckExistsParam param) {
        ExistsMemberVo vo = new ExistsMemberVo();
        String email = param.getEmail();
        String name = param.getName();
        String phone = param.getPhone();
        MembersExample example = new MembersExample();
        MembersExample.Criteria criteria = example.createCriteria();
        if(!StringUtils.isEmpty(email)){
            criteria.andEmailEqualTo(email);
        }else if(!StringUtils.isEmpty(name)){
            criteria.andNameEqualTo(name);
        }else{
            criteria.andPhoneEqualTo(phone);
        }
        List<Members> list = membersMapper.selectByExample(example);
        if(!CollectionUtils.isEmpty(list)){
            Members members = list.get(0);
            if(!StringUtils.isEmpty(email)){
                vo.setKey("eamil");
                vo.setValue(members.getEmail());
            }else if(!StringUtils.isEmpty(name)){
                vo.setKey("name");
                vo.setValue(members.getName());
            }else{
                vo.setKey("phone");
                vo.setValue(members.getPhone());
            }
        }
        return vo;
    }
}
