package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.client.ComPdfKitClient;
import cn.kdan.pdf.backend.core.constant.MissionConstant;
import cn.kdan.pdf.backend.core.enums.MissionFileStatusEnum;
import cn.kdan.pdf.backend.core.enums.SubscriberTypeEnum;
import cn.kdan.pdf.backend.core.mapper.MissionsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.model.member.MemberInfo;
import cn.kdan.pdf.backend.core.params.MissionCreateParam;
import cn.kdan.pdf.backend.core.pojo.ConvertTypeVo;
import cn.kdan.pdf.backend.core.pojo.MissionFileVO;
import cn.kdan.pdf.backend.core.pojo.MissionVO;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.CreateTaskResult;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.FileInfoDto;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.QueryTaskInfoResult;
import cn.kdan.pdf.backend.core.pojo.comPdfKit.WebhookSendDTO;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.ComPdfKitUtils;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;
import utils.DateUtils;
import utils.DistributedLocker;
import utils.JsonUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class MissionServiceImpl implements MissionService {

    @Resource
    MissionsMapper missionsMapper;
    @Resource
    MemberService memberService;
    @Resource
    ComPdfKitUtils comPdfKitUtils;
    @Resource
    MissionFilesService missionFilesService;
    @Resource
    ConvertFileService convertFileService;
    @Resource
    private DistributedLocker distributedLocker;
    @Resource
    private OutputFileService outputFileService;

    /**
     * saas任务完成状态
     */
    private final static String TASK_FINISH = "TaskFinish";
    private final static String TASK_PROCESSING = "TaskProcessing";


    @Transactional(rollbackFor = BackendRuntimeException.class)
    @Override
    public MissionVO create(List<MissionCreateParam> paramList) {
        //生成对应mission（任务）记录：因为不同的文件类型转换在saas需要创建不同的任务去处理，
        // 而17官网一次任务可能有不同的类型，因此一个mission在saas需要生成几个不同的任务去处理，
        // mission表中以（1，2，3）的形式去记录多个saas的任务id
        //如果文件列表为空抛出异常
        if(CollectionUtils.isEmpty(paramList)){
            throw new BackendRuntimeException(MissionConstant.EXCEPTION_MSG_EMPTY_PARAMS);
        }
        MemberInfo memberInfo = memberService.getMemberDetail(null).getMemberInfo();
        //获取对应转档类型的耗券集合
        Map<String,ConvertTypes>  convertTypesMap = getConvertTypeMap();
        //判断用户是否是会员，如果不是会员的话判断用户的券是否足够
        int totalPrice = handleConsumePoint(paramList, memberInfo, convertTypesMap);
        ComPdfKitClient client = comPdfKitUtils.getComPdfKitClient();
        String missionId = CommonUtils.generateId();
        //获取不同类型的taskId
        Map<String,String> fileTypeTaskIdMap = handleFileType(paramList,client);
        List<String> saasTaskIds = new ArrayList<>(fileTypeTaskIdMap.values());
        List<MissionFileVO> missionFilePoJos = new ArrayList<>();
        //插入文件表
        for (MissionCreateParam param : paramList) {
            String executeType = getExecuteType(param.getInput(), param.getOutput());
            MissionFiles missionFile = new MissionFiles();
            missionFile.setId(CommonUtils.generateId());
            missionFile.setMemberId(memberInfo.getId());
            missionFile.setMissionId(missionId);
            missionFile.setFileName(param.getFilename());
            missionFile.setOriginalFilename(param.getFilename());
            missionFile.setSize(param.getSize());
            missionFile.setSourceType(param.getSourceType());
            missionFile.setCreatedAt(new Date());
            missionFile.setUpdatedAt(new Date());
            ConvertTypes convertType = convertTypesMap.get(executeType);
            missionFile.setConvertTypeId(convertType.getId());
            Integer filePrice = SubscriberTypeEnum.yes.value().equals(memberInfo.getSubscriberType()) ? convertType.getSpPrice() : convertType.getPrice();
            //根据转换类型获取saas任务id
            missionFile.setPrice(filePrice);
            missionFile.setSaasTaskId(fileTypeTaskIdMap.get(executeType));
            missionFile.setStatus(MissionFileStatusEnum.CREATED.value());
            // modify 转档任务价格设置逻辑修改，无须再加上每个文件的价格 2022年11月28日15:51:19
//            totalPrice = totalPrice + filePrice;
            missionFilesService.insert(missionFile);
            //插入list返回前端
            MissionFileVO missionFileVO = new MissionFileVO();
            BeanUtils.copyProperties(missionFile, missionFileVO);
            missionFileVO.setOutputType(param.getOutput());
            missionFilePoJos.add(missionFileVO);
        }
        //插入任务表
        Missions mission = new Missions()
                .withId(missionId)
                .withCreatedAt(new Date())
                .withClientId(null)
                .withPrice(totalPrice)
                .withMemberId(memberInfo.getId())
                .withUpdatedAt(new Date())
                .withSaasTaskIds(saasTaskIds.toString());
        missionsMapper.insert(mission);
        MissionVO pojo = new MissionVO();
        BeanUtils.copyProperties(mission,pojo);
        pojo.setMissionFilePoJos(missionFilePoJos);
        return pojo;
    }


    /**
     * 根据用户的会员状态对用户进行预扣费，最后返回一次任务总耗券数
     *
     * @param paramList paramList
     * @param memberInfo memberInfo
     * @param convertTypesMap convertTypesMap
     * @return  totalPrice
     */
    private int handleConsumePoint(List<MissionCreateParam> paramList, MemberInfo memberInfo, Map<String, ConvertTypes> convertTypesMap) {
        int totalPrice = 0 ;
        //如果不是会员,则对用户进行预扣券操作
        if(SubscriberTypeEnum.no.value().equals(memberInfo.getSubscriberType())){
            totalPrice = getTotalPrice(paramList, memberInfo.getSubscriberType(), convertTypesMap);
            if(memberInfo.getPoints()<totalPrice){
                //没有足够的券
                throw new BackendRuntimeException(MissionConstant.EXCEPTION_MSG_NO_ENOUGH_CREDIT);
            }
        }
        return totalPrice;
    }

    /**
     * 获取总耗券数
     * @param paramList paramList
     * @param subscriberType subscriberType
     * @param convertTypesMap convertTypesMap
     * @return 获取总耗券数
     */
    private int getTotalPrice(List<MissionCreateParam> paramList, Integer subscriberType, Map<String, ConvertTypes> convertTypesMap) {
        int totalPrice = 0;
        for (MissionCreateParam param : paramList) {
            String executeType = getExecuteType(param.getInput(), param.getOutput());
            ConvertTypes convertType = convertTypesMap.get(executeType);
            Integer filePrice = SubscriberTypeEnum.yes.value().equals(subscriberType) ? convertType.getSpPrice() : convertType.getPrice();
            totalPrice = totalPrice + filePrice;
        }
        return totalPrice;
    }

    /**
     * 根据不同的转档类型调佣saas生成不同的任务，最后生成一个<转档类型，saasTaskId>形式的map方便取用
     *
     * @param paramList paramList
     * @param client client
     * @return Map
     */
    private Map<String,String> handleFileType (List<MissionCreateParam> paramList, ComPdfKitClient client) {
        Map<String,String> map = new HashMap<>();
        paramList.forEach(item->{
            String executeType = getExecuteType(item.getInput(),item.getOutput());
            if(!map.containsKey(executeType)){
                CreateTaskResult createTaskResult = client.createTask(executeType);
                map.put(executeType,createTaskResult.getTaskId());
            }
        });
        return map;
    }

    /**
     * 获取用户转档类型 （pdf/docx）
     *
     * @param inPut 输入
     * @param outPut 输出
     * @return String
     */
    private String getExecuteType (String inPut,String outPut) {
        return inPut.concat(CommonConstant.STRING_VIRGULE).concat(outPut);
    }

    /**
     *  查询出所有的转档类型，放到
     *
     * @return Map
     */
    private Map<String,ConvertTypes> getConvertTypeMap () {
        Map<String,ConvertTypes> map = new HashMap<>();
       List<ConvertTypes> list =  convertFileService.getAllConvertType();
        list.forEach(item->{
            String executeType =  getExecuteType(item.getInputType(),item.getOutputType());
            if(!map.containsKey( executeType)){
                map.put(executeType,item);
            }
        });
       return map;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public String convertFile(String missionId) {
        /*
            3.所有文件上传成功之后（即所有上传文件接口返回是完成）调用转档接口
            （1）转档根据mission中储存的任务id去调用saas服务开始转档
            （2）前端根据missionId查询mission任务信息，根据任务表中的saas任务id去分别查询saas服务文件转档状态，
                如果任务为finish，则判断任务中的文件为成功或者是失败，修改本地文件表状态
            （3 异步回调：根据saas文件id修改本地文件表中的对应的文件信息和状态
         */

        //（1）转档根据mission中储存的任务id去调用saas服务开始转档
        MemberInfo memberInfo = memberService.getMemberDetail(null).getMemberInfo();
        String memberId = memberInfo.getId();
        //要保证查询用户的余券和预扣券是原子的，因此加锁
        String lockKey = MissionConstant.LOCK_KEY + memberId ;
        Missions mission = missionsMapper.selectByPrimaryKey(missionId);
        distributedLocker.lock(lockKey,60);
        //获取对应转档类型的耗券集合
        //判断用户是否是会员，如果不是会员的话判断用户的券是否足够，并且对用户进行预扣券
        int totalPrice = mission.getPrice();
        if(SubscriberTypeEnum.no.value().equals(memberInfo.getSubscriberType())){
            if(memberInfo.getPoints()<totalPrice){
                //没有足够的券
                throw new BackendRuntimeException(MissionConstant.EXCEPTION_MSG_NO_ENOUGH_CREDIT);
            }
            Members member = new Members();
            member.setId(memberInfo.getId());
            member.setPoints(memberInfo.getPoints() - totalPrice);
            memberService.update(member);
        }
        //预扣券完毕，释放锁
        distributedLocker.unlock(lockKey);
        String[] saasTaskIds = mission.getSaasTaskIds().replaceAll("[\\[\\]]", "").split(",");
        // 获取saas服务
        ComPdfKitClient client = comPdfKitUtils.getComPdfKitClient();

        for (String id : saasTaskIds) {
            // 更新missionFile转档开始时间
            log.info("调用saas转档接口开始，missionId：{}",missionId);
            missionFilesService.updateBySaasTaskId(id.trim());
            // 开启转档
            CreateTaskResult taskId = client.executeTask(id.trim());
            log.info("调用saas转档接口成功，taskId：{}",taskId);
        }
        return missionId;
    }

    @Override
    @Transactional(rollbackFor = BackendRuntimeException.class)
    public List<MissionVO> queryFileStatus(String missionId){
        Members members = memberService.getCurrentUser();
        //查询出任务信息
        Missions mission = missionsMapper.selectByPrimaryKey(missionId);
        List<String> saasTaskIds;
        if(!StringUtils.isEmpty(mission.getSaasTaskIds())) {
            saasTaskIds = Arrays.asList(mission.getSaasTaskIds().replaceAll("[\\[\\]]", "").split(","));
        }else{
            throw new BackendRuntimeException("当前任务未关联到saas任务，missionId："+missionId);
        }
        ComPdfKitClient client = comPdfKitUtils.getComPdfKitClient();
        for (String taskId : saasTaskIds) {
            querySaasToUpdateStatus(members, client, taskId.trim());
        }
        List<MissionVO> fileStatus = convertFileService.getFileStatus(missionId);
        return fileStatus;
    }

    /**
     * 查询saas转档任务状态并进行missionFile表修改
     *
     * @param members 用户
     * @param taskId saas任务id
     */
    private void querySaasToUpdateStatus(Members members, ComPdfKitClient client, String taskId) {
        QueryTaskInfoResult result = client.queryTaskInfo(taskId);
        String taskStatus = result.getTaskStatus();
        List<FileInfoDto> list = result.getFileInfoDTOList();
        // 判断文件是成功还是失败,任务状态为TaskFinish或者TaskProcessing时，修改文件状态
        if(TASK_FINISH.equals(taskStatus) || TASK_PROCESSING.equals(taskStatus)){
            for (FileInfoDto file : list) {
                MissionFiles missionFiles = missionFilesService.queryByProcessId(file.getFileKey());
                if("success".equals(file.getStatus())){
                    if(!missionFiles.getStatus().equals(MissionFileStatusEnum.SUCCESS.value())) {
                        //成功的话修改本地missionFile表状态
                        //修改missionFile对象信息（convert_end_at,convert_at）
                        missionFiles.setOutFilename(file.getDownFileName());
                        missionFiles.setPath(file.getDownloadUrl());
                        missionFiles.setConvertEndAt(new Date());
                        missionFiles.setStatus(MissionFileStatusEnum.SUCCESS.value());
                        missionFiles.setUpdatedAt(new Date());
                        missionFilesService.update(missionFiles);
                        OutputFiles outputFile = new OutputFiles();
                        outputFile.setId(CommonUtils.generateId());
                        outputFile.setMissionId(missionFiles.getMissionId());
                        outputFile.setMemberId(members.getId());
                        outputFile.setMissionFileId(missionFiles.getId());
                        outputFile.setFileName(missionFiles.getOutFilename());
                        outputFile.setOriginalFilename(missionFiles.getOriginalFilename());
                        outputFile.setPath(missionFiles.getPath());
                        outputFile.setSize(Integer.valueOf(file.getConvertSize()));
                        outputFileService.insert(outputFile);
                    }
                }else if ("failed".equals(file.getStatus())){
                    //先查询文件状态是否已更新成失败状态，已更新则不返回券数
                    if(!MissionFileStatusEnum.FAILED.value().equals(missionFiles.getStatus())){
                        //修改文件状态为失败
                        fileFailUpdate(members, file, missionFiles,false);
                    }
                } else{
                    // 如果文件超时15分钟，将文件设置为转档失败
                    long convertTime = DateUtils.calMinutesToNow(missionFiles.getConvertAt());
                    if(convertTime > 15){
                        fileFailUpdate(members,file,missionFiles,true);
                    }
                }
            }
        }
    }

    /**
     * 失败文件处理，更新文件状态，退券处理
     * @param members 用户信息
     * @param file saas返回文件信息
     * @param missionFiles 文件信息
     * @param flag 是否超时
     */
    private void fileFailUpdate(Members members, FileInfoDto file, MissionFiles missionFiles, boolean flag) {
        missionFiles.setStatus(MissionFileStatusEnum.FAILED.value());
        if(flag){
            missionFiles.setFailReason("文件转换失败：文件转换超时");
        }else{
            missionFiles.setFailReason(file.getFailureReason());
        }
        missionFiles.setPrice(0);
//        missionFiles.setSize(BigDecimal.ZERO);
        missionFiles.setUpdatedAt(new Date());
        missionFiles.setConvertEndAt(new Date());
        missionFilesService.update(missionFiles);
        //失败的话，判断是否是会员，不是会员的情况下要返回失败扣的券数
        if(SubscriberTypeEnum.no.value().equals(members.getSubscriberType())){
            // 返券操作
            List<ConvertTypeVo> convertTypeList = convertFileService.getConvertTypeList(file.getSourceType(), file.getTargetType());
            Integer number = 0;
            if(!CollectionUtils.isEmpty(convertTypeList)){
                number = convertTypeList.get(0).getPrice();
            }
            members.setPoints(members.getPoints()+number);
            memberService.update(members);
        }
    }

    /**
     * saas回调函数
     *
     * @param dto saas回调参数
     */
    @Override
    public void saasCallback(WebhookSendDTO dto){
        log.info("开始进行回调,回调参数：{}", JsonUtils.getJsonString(dto));
        Object eventObject = dto.getEventObject();
        if (ObjectUtils.isEmpty(eventObject)) {
            log.info("Saas回调失败，本次事件的对象为空");
            return;
        }
        //该id即为saasTaskId（没理解错的话）
        String id = eventObject.toString();
        //不采用mission查看
        List<MissionFiles> list = missionFilesService.getBySaasId(id);
        if (CollectionUtils.isEmpty(list)) {
            log.info("未查询到任务,saasTaskId：{}",id);
            return;
        }
        //获取用户对象
        String memberId = list.get(0).getMemberId();
        Members members = memberService.getById(memberId);
        ComPdfKitClient client = comPdfKitUtils.getComPdfKitClient();
        //查询saas任务状态并进行missionFile表修改
        querySaasToUpdateStatus(members,client,id);
        log.info("回调结束");
    }

}
