package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.OrderConstant;
import cn.kdan.pdf.backend.core.constant.SubscriptionConstant;
import cn.kdan.pdf.backend.core.enums.*;
import cn.kdan.pdf.backend.core.mapper.OrdersMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.pojo.OrderVO;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.alipay.AlipayNotifyParam;
import cn.kdan.pdf.backend.core.utils.wxpay.*;
import com.alibaba.fastjson.JSON;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayConfig;
import com.alipay.api.internal.util.AlipaySignature;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.catalina.util.Strftime;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;
import utils.DateUtils;
import utils.DistributedLocker;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.*;


/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class OrderServiceImpl implements OrderService {

    @Autowired
    private OrdersMapper ordersMapper;
    @Autowired
    private SetPricingService setPricingService;
    @Autowired
    private PricingService pricingService;
    @Autowired
    private WXPayService wxPayService;
    @Autowired
    private SubscriptionService subscriptionService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private DistributedLocker distributedLocker;
    @Autowired
    private AlipayService alipayService;
    @Autowired
    private Environment config;
    @Autowired
    private MyConfig myWxConfig;

    //支付宝查询返回参数名
    private final static String RESPONSE = "alipay_trade_query_response";
    //交易状态
    private final static String TRADE_STATUS = "trade_status";


    private static final ThreadPoolExecutor PAY_NOTIFY_THREAD_POOL = new ThreadPoolExecutor( Runtime.getRuntime().availableProcessors(),
            Runtime.getRuntime().availableProcessors(),
            1L,
            TimeUnit.MINUTES,
            new LinkedBlockingQueue<>(300), new ThreadFactoryBuilder()
            .setNameFormat("PAY_NOTIFY_THREAD_POOL").build(), new ThreadPoolExecutor.AbortPolicy());


    /**
     * 根据订阅id查询订单
     * @param subId 订阅id
     * @return 订单
     */
    public Orders selectBySubId(String subId){
        OrdersExample example = new OrdersExample();
        example.createCriteria().andSubscriptionIdEqualTo(subId);
        List<Orders> orders = ordersMapper.selectByExample(example);
        if(!CollectionUtils.isEmpty(orders)){
            return orders.get(0);
        }
        return null;
    }

    @Override
    public Orders findMemberUnPayedOrders(String memberId, Subscriptions subscription, String client){
        OrdersExample example = new OrdersExample();
        OrdersExample.Criteria criteria = example.createCriteria();
        //如果是web端，则查询的pagePayUrl不为null；如果是安卓端，则查询的androidSign不为null。
        if(ClientEnum.WEB.value().equals(client)) {
            criteria.andPagePayUrlIsNotNull();
        }else{
            criteria.andAndroidSignIsNotNull();
        }
        criteria.andMemberIdEqualTo(memberId).andStatusEqualTo(OrdersStatusEnum.UNPAYED.value()).
                andPaymentEqualTo(subscription.getPayment()).andSubscriptionIdEqualTo(subscription.getId());
        List<Orders> list  = ordersMapper.selectByExample(example);
        //订单如果不存在则插入，存在则返回
        if(CollectionUtils.isEmpty(list)) {
            Orders order = new Orders();
            order.setId(CommonUtils.generateId());
            order.setMemberId(memberId);
            order.setSubscriptionId(subscription.getId());
            order.setPayment(subscription.getPayment());
            order.setStatus(OrdersStatusEnum.UNPAYED.value());
            order.setCreatedAt(new Date());
            order.setUpdatedAt(new Date());
            order.setPrice(subscription.getPrice());
            //设置支付的url
            setOrderPayUrlByPayment(subscription, order, client);
            //插入
            ordersMapper.insert(order);
            return order;
        }else{
            //更新价格
            Orders order = selectBySubId(subscription.getId());
            // 由于存在安卓端和web端，不能单纯的靠价格来判断是否更新，安卓端需要的是androidSign，web需要的是pagePayUrl
            if(order != null && !order.getPrice().equals(subscription.getPrice())) {
                order.setPrice(subscription.getPrice());
                order.setCreatedAt(new Date());
                order.setUpdatedAt(new Date());
                //设置支付的url
                setOrderPayUrlByPayment(subscription, order, client);
                ordersMapper.updateByPrimaryKey(order);
            }
        }
        //如果微信url为空或者微信url超过两小时过期了，则重重新生成
        boolean wxCheckDateFlag = StringUtils.isEmpty(list.get(0).getQrcodeUrl()) || DateUtils.addHours(list.get(0).getUpdatedAt(),OrderConstant.WXPAY_EXPIRE_TIME,1).compareTo(new Date())<0;
        boolean aliCheckDateFlag = StringUtils.isEmpty(list.get(0).getPagePayUrl()) || DateUtils.addHours(list.get(0).getUpdatedAt(),OrderConstant.ALIPAY_EXPIRE_TIME,1).compareTo(new Date())<0;
        boolean wxResetFlag = PaymentEnum.WXPAY.value().equals(subscription.getPayment()) && wxCheckDateFlag;
        boolean aliResetFlag = PaymentEnum.ALIPAY.value().equals(subscription.getPayment()) && aliCheckDateFlag;
        if(wxResetFlag||aliResetFlag){
            //处理必要的订单号 支付主题等数据
            resetPayUrl(subscription, list.get(0), client);
        }
       return list.get(0);
    }

    /**
     * 生成新的支付url 并且更新时间
     * @param subscription 订阅
     * @param order 订单
     */
    private void resetPayUrl(Subscriptions subscription, Orders order, String client) {
        setOrderPayUrlByPayment(subscription, order, client);
        order.setUpdatedAt(new Date());
        ordersMapper.updateByPrimaryKey(order);
    }

    /**
     * 根据不同的支付类型设置不同url
     * @param subscription 订阅
     * @param order 订单
     */
    private void setOrderPayUrlByPayment(Subscriptions subscription, Orders order, String client) {
        String subject = handleTradeNoAndSubject(subscription, order);
        if(PaymentEnum.WXPAY.value().equals(subscription.getPayment())){
            Map<String, String> resp = wxPayService.unifiedOrder(subject, order.getTradeNo(), order.getPrice(), client);
            order.setQrcodeUrl(resp.get(WXPayConstants.CODE_URL));
            order.setAndroidSign(JSON.toJSONString(resp));
        }
        if(PaymentEnum.ALIPAY.value().equals(subscription.getPayment())){
            // 调用阿里支付下单接口,web端返回支付url，android端返回androidSign
            String alipayResult = alipayService.createAlipayPageUrl(subject, order.getTradeNo(), order.getPrice(), subscription, client);
            if(ClientEnum.WEB.value().equals(client)) {
                order.setPagePayUrl(alipayResult);
            }else {
                order.setAndroidSign(alipayResult);
            }
        }
    }

    /**
     * 生成新的订单号和支付主题
     * @param subscription 订阅 用于填充新的订单号
     * @param order 订单
     * @return 支付主题
     */
    private String handleTradeNoAndSubject(Subscriptions subscription, Orders order) {
        String subject ;
        if (GoodsTypeEnum.SetPricing.value().equals(subscription.getTargetType())) {
            SetPricings setPricing = setPricingService.getById(subscription.getTargetId());
            subject = initSubject (subscription.getTargetType(),setPricing.getTitle());
            order.setTradeNo(autoTradeNo(subscription.getTargetType(),setPricing.getMode().toString()));
        } else {
            Pricings pricing = pricingService.getById(subscription.getTargetId());
            subject = initSubject (subscription.getTargetType(),pricing.getTitle());
            order.setTradeNo(autoTradeNo(subscription.getTargetType(),pricing.getMode().toString()));
        }
        return subject;
    }

    @Override
    public Orders getByTradeNo(String outTradeNo) {
        OrdersExample example = new OrdersExample();
        example.createCriteria().andTradeNoEqualTo(outTradeNo);
        List<Orders> orders = ordersMapper.selectByExample(example);

        return CollectionUtils.isEmpty(orders)? null : orders.get(0);
    }

    /**
     * 获取订单编号
     */
    private String autoTradeNo (String targetType, String targetMode) {
        Strftime strftime = new Strftime( "%Y%m%d%H%M", Locale.CHINESE );
        return strftime.format(new Date()) + getOrderTypeNo(targetType,targetMode) + (System.currentTimeMillis()+"").substring(9) ;
    }


    /**
     * 根据订单的类型和状态获取自动的订单号
     */
    private  String getOrderTypeNo (String targetType,String targetMode) {
        String orderTypeNo = "00";
        switch (targetType) {
            case "Pricing":
                orderTypeNo  = "01";
                break;
            case "SetPricing":
                if("yearly_base".equals(targetMode)){
                    orderTypeNo  =  "03";
                } else if ("monthly_base".equals(targetMode)) {
                    orderTypeNo  =  "04";
                } else if ("yearly_business".equals(targetMode)) {
                    orderTypeNo  =  "05";
                } else if ("monthly_business".equals(targetMode)) {
                    orderTypeNo  =  "06";
                }
                break;
            default:  orderTypeNo  = "00";
                break;
        }
        return  orderTypeNo;
    }

    private String initSubject (String targetType,String title) {
        if(GoodsTypeEnum.SetPricing.value().equals(targetType)){
            return "17PDF会员订阅"+title;
        }else{
            return "17PDF会员购买"+title;
        }
    }

    @Override
    public void handleWxOrder(String tradeNo){
        PAY_NOTIFY_THREAD_POOL.submit(() -> {
            //  订单号
            while (true) {
                String lockKey = "WX_ORDER_LOCK_KEY_" + tradeNo ;
                Orders order = checkOrders(tradeNo, lockKey);
                if (ObjectUtils.isEmpty(order)) {
                    break;
                }
                //查询订单状态根据订单不同的状态，关闭订单，更新订单表,更新订阅表,更新用户状态
                handlePayResult(order);
                //释放锁
                distributedLocker.unlock(lockKey);
                break;
            }
        });
    }

    private Orders checkOrders(String tradeNo, String lockKey) {
        distributedLocker.lock(lockKey,60);
        Orders order = getByTradeNo(tradeNo);
        //如果订单不存在
        if(ObjectUtils.isEmpty(order)){
            log.error(SubscriptionConstant.EXCEPTION_MSG_INVALID_TRADE_NO);
            return null;
        }
        //如果订单已经处理过,则不再处理
        if(!OrdersStatusEnum.UNPAYED.value().equals(order.getStatus())){
            log.info(SubscriptionConstant.EXCEPTION_MSG_REPEAT_TRADE);
            return null;
        }
        return order;
    }

    /**
     * 查询微信订单，根据结果更新数据
     */
    private void handlePayResult(Orders order){
        WXPayOrderQueryResult wxPayOrderQueryResult = wxPayService.orderQuery(order.getTradeNo());
        switch(WxOrderStatusEnum.getByCode(wxPayOrderQueryResult.getTradeState())) {
            case NOT_PAY:
                //如果是未支付，记录result
                updateOrder(order,wxPayOrderQueryResult.toString(),OrdersStatusEnum.UNPAYED.value());
                break;
            case CLOSED:
            case REVOKED:
            case USER_PAYING:
            case PAYERROR:
                updateOrder(order,wxPayOrderQueryResult.toString(),OrdersStatusEnum.CLOSED.value());
                handleWxPayOrderClosed(order);
                break;
            case SUCCESS:
                //更新订单状态
                updateOrder(order,wxPayOrderQueryResult.toString(),OrdersStatusEnum.SUCCESS.value());
                //更新订阅表为生效
                //用户总时间，券数，以及会员状态
                handlePayOrderSuccess(order);
                break;
            case REFUND:
                updateOrder(order,wxPayOrderQueryResult.toString(),OrdersStatusEnum.CLOSED.value());
                //退费
                handleWxPayOrderRefund(order);
                break;
            default:
                log.warn("Warn wxpay: "+ wxPayOrderQueryResult);
                break;
        }
    }

    private void updateOrder(Orders order,String result,Integer status){
        order.setResult(result);
        order.setStatus(status);
        ordersMapper.updateByPrimaryKey(order);
    }

    private void handleWxPayOrderClosed(Orders order){
        //关闭订单后更新订阅表
        Subscriptions subscription = new Subscriptions();
        subscription.setId(order.getSubscriptionId());
        subscription.setStatus(SubscriptionStatusEnum.UNPAYED.value());
        subscriptionService.update(subscription);
    }


    private void handleWxPayOrderRefund(Orders order){
        //关闭订单后更新订阅表
        Subscriptions subscription = new Subscriptions();
        subscription.setId(order.getSubscriptionId());
        subscription.setStatus(SubscriptionStatusEnum.UNPAYED.value());
        subscriptionService.update(subscription);
    }

    /**
     * 处理微信支付成功之后的逻辑
     * @param order order
     */
    private void handlePayOrderSuccess(Orders order){
        //更新订阅表为生效，并且重新计算时间
        Subscriptions subscription = subscriptionService.getById(order.getSubscriptionId());
        if(subscription.getStatus().equals(SubscriptionStatusEnum.ACTIVED.value())){
            throw new BackendRuntimeException(OrderConstant.EXCEPTION_MSG_GET_ORDER_EXIST);
        }
        subscription.setStatus(SubscriptionStatusEnum.ACTIVED.value());
        subscriptionService.update(subscription);
        //用户总时间，券数，以及会员状态
        //是充值券还是充值,两者都会计算券，如果是会员还会计算时间
        Members members = memberService.getById(order.getMemberId());
        if(GoodsTypeEnum.SetPricing.value().equals(subscription.getTargetType())){
            //修改用户会员标志
            members.setSubscriberType(SubscriberTypeEnum.yes.value());
            //更新订阅时间
            subscriptionService.renewInfo(true,ClientEnum.WEB.value(),subscription,members,setPricingService.getById(subscription.getTargetId()));
        }
        //修改用户当前券数及总券数
        members.setTotalPoints(members.getTotalPoints()+subscription.getTotalPoints());
        members.setPoints(members.getPoints()+subscription.getTotalPoints());
        memberService.update(members);
    }

    @Override
    public OrderVO getWxOrders(String orderId) {
        Orders order = ordersMapper.selectByPrimaryKey(orderId);
        OrderVO orderVO = new OrderVO();
        //如果订单不存在
        if(ObjectUtils.isEmpty(order)){
            throw new BackendRuntimeException(SubscriptionConstant.EXCEPTION_MSG_INVALID_TRADE_NO);
        }
        //如果订单已经处理过，则不再处理（如果订单状态为其他则都表示已经处理过了）
        if(OrdersStatusEnum.UNPAYED.value().equals(order.getStatus())){
            String lockKey = "WX_ORDER_LOCK_KEY_" + order.getTradeNo() ;
            distributedLocker.lock(lockKey,60);
            //查询订单状态根据订单不同的状态，关闭订单，更新订单表,更新订阅表,更新用户状态
            handlePayResult(order);
            //释放锁
            distributedLocker.unlock(lockKey);
            BeanUtils.copyProperties(ordersMapper.selectByPrimaryKey(orderId), orderVO);
            orderVO.setStatusName(OrdersStatusEnum.getNameByValue(order.getStatus()));
            //更新完状态之后再次查询一次
            return orderVO;
        }
        //隐藏result（里面有商户配置）
        BeanUtils.copyProperties(order, orderVO);
        orderVO.setStatusName(OrdersStatusEnum.getNameByValue(order.getStatus()));
        return orderVO;
    }
    @Override
    public String syncOrder(String xmlData) {
        // 支付结果通知的xml格式数据
        WXPay wxpay = new WXPay(myWxConfig);
        // 转换成map
        Map<String, String> notifyMap ;
        try {
            notifyMap = WXPayUtil.xmlToMap(xmlData);
            WXPayOrderNotifyResult result = JSON.parseObject(JSON.toJSONString(notifyMap), WXPayOrderNotifyResult.class);
            if (wxpay.isPayResultNotifySignatureValid(notifyMap)) {
                // 签名正确，进行处理。
                // 注意特殊情况：订单已经退款，但收到了支付结果成功的通知，不应把商户侧订单状态从退款改成支付成功
                if(StringUtils.isEmpty(result.getOutTradeNo())){
                    //回应微信
                    log.error(SubscriptionConstant.EXCEPTION_MSG_INVALID_TRADE_NO);
                    return wxResponse(Boolean.FALSE, SubscriptionConstant.EXCEPTION_MSG_INVALID_TRADE_NO);
                }
                //支付成功
                if(result.getResultCode().equals(WXPayConstants.SUCCESS)&&result.getReturnCode().equals(WXPayConstants.SUCCESS)){
                    //考虑并发，分布式情况下，有一个线程处理的比较慢的话就可能会有另一个线程处理微信的回调导致给用户加上额外的时间，根据redis的状态判断是否已经有线程在处理该订单
                    handleWxOrder(result.getOutTradeNo());
                    return wxResponse(Boolean.TRUE, CommonConstant.CODE_OK);
                }else{
                    //如果支付失败
                    log.error(notifyMap.toString());
                    return wxResponse(Boolean.FALSE,notifyMap.toString());
                }
            }
            else {
                // 签名错误，如果数据里没有sign字段，也认为是签名错误
                return wxResponse(Boolean.FALSE,"伪造的通知!");
            }
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new RuntimeException(e);
        }
    }


    private String wxResponse (Boolean isSuccess,String msg) {
        return getResponse(isSuccess, msg);
    }

    private String getResponse(Boolean isSuccess, String msg) {
        Map<String,String> map  = new HashMap<>();
        map.put("return_code",isSuccess? WXPayConstants.SUCCESS:WXPayConstants.FAIL);
        map.put("return_msg",msg);
        try {
            return WXPayUtil.mapToXml(map);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }

    /**
     * 第一步:验证签名,签名通过后进行第二步
     * 第二步:按一下步骤进行验证
         * 1、商户需要验证该通知数据中的out_trade_no是否为商户系统中创建的订单号，
         * 2、判断total_amount是否确实为该订单的实际金额（即商户订单创建时的金额），
         * 3、校验通知中的seller_id（或者seller_email) 是否为out_trade_no这笔单据的对应的操作方（有的时候，一个商户可能有多个seller_id/seller_email），
         * 4、验证app_id是否为该商户本身。上述1、2、3、4有任何一个验证不通过，则表明本次通知是异常通知，务必忽略。
     * 在上述验证通过后商户必须根据支付宝不同类型的业务通知，正确的进行不同的业务处理，并且过滤重复的通知结果数据。
     * 在支付宝的业务通知中，只有交易通知状态为TRADE_SUCCESS或TRADE_FINISHED时，支付宝才会认定为买家付款成功。
     *
     * @param request 支付宝
     * @return 回调结果
     */
    @Override
    public String alipaySyncOrder(HttpServletRequest request) {
        // 将异步通知中收到的待验证所有参数都存放到map中
        Map<String, String> params = convertRequestParamsToMap(request);
        String paramsJson = JSON.toJSONString(params);
        log.info("支付宝回调，{}", paramsJson);

        try {
            // 支付宝配置
            AlipayConfig alipayConfig = new AlipayConfig();
            alipayConfig.setAlipayPublicKey(config.getProperty("ali.pay.publicKey"));
            alipayConfig.setServerUrl(config.getProperty("ali.pay.gatewayUrl"));
            alipayConfig.setAppId(config.getProperty("ali.pay.appId"));
            alipayConfig.setPrivateKey(config.getProperty("ali.pay.rsa2Key"));
            // 调用SDK验证签名
            boolean signVerified = AlipaySignature.rsaCheckV1(params, alipayConfig.getAlipayPublicKey(),
                    alipayConfig.getCharset(), alipayConfig.getSignType());
            if (signVerified) {
                log.info("支付宝回调签名认证成功");
                // 按照支付结果异步通知中的描述，对支付结果中的业务内容进行1\2\3\4二次校验，校验成功后在response中返回success，校验失败返回failure
                this.check(params,alipayConfig);
                // 另起线程处理业务
                handleAliOrder(params, paramsJson);
                // 如果签名验证正确，立即返回success，后续业务另起线程单独处理
                // 业务处理失败，可查看日志进行补偿，跟支付宝已经没多大关系。
                return "success";
            } else {
                log.info("支付宝回调签名认证失败，signVerified=false, paramsJson:{}", paramsJson);
                return "fail";
            }
        } catch (AlipayApiException e) {
            log.error("支付宝回调签名认证失败,paramsJson:{},errorMsg:{}", paramsJson, e.getMessage());
            return "fail";
        }
    }

    private String aliResponse (Boolean isSuccess,String msg) {
        return getResponse(isSuccess, msg);
    }


    /**
     * 处理支付宝订单
     *
     * @param params params
     * @param paramsJson paramJson
     */
    private void handleAliOrder(Map<String, String> params, String paramsJson) {
        PAY_NOTIFY_THREAD_POOL.execute(() -> {
            while(true){
                AlipayNotifyParam param = buildAlipayNotifyParam(params);
                String outTradeNo = param.getOutTradeNo();
                String lockKey = "ALI_ORDER_LOCK_KEY_" + outTradeNo;
                Orders order = checkOrders(outTradeNo, lockKey);
                if (ObjectUtils.isEmpty(order)) {
                    break;
                }
                String result = alipayService.query(outTradeNo);
                log.info("支付宝回调查询结果：{}",result);
                Map<String,Object> map = JSON.parseObject(result, Map.class);
                Map<String,String> response = JSON.parseObject(map.get(RESPONSE).toString(), Map.class);
                String tradeStatus = response.get(TRADE_STATUS);
                //查询订单状态根据订单不同的状态，关闭订单，更新订单表,更新订阅表,更新用户状态
                // 支付成功
                if (AlipayTradeStatus.TRADE_SUCCESS.getStatus().equals(tradeStatus)
                        || AlipayTradeStatus.TRADE_FINISHED.getStatus().equals(tradeStatus)) {
                    // 处理支付成功逻辑
                    try {
                        if(AlipayTradeStatus.TRADE_SUCCESS.getStatus().equals(tradeStatus)) {
                            updateOrder(order, result, OrdersStatusEnum.SUCCESS.value());
                        } else{
                            updateOrder(order, result, OrdersStatusEnum.FINISH.value());
                        }
                        //订单成功后的操作
                        handlePayOrderSuccess(order);
                    } catch (Exception e) {
                        log.error("支付宝回调业务处理报错,params:" + paramsJson, e);
                    }
                } else {
                    if(AlipayTradeStatus.TRADE_CLOSED.getStatus().equals(tradeStatus)){
                        updateOrder(order,result,OrdersStatusEnum.CLOSED.value());
                    }else if(AlipayTradeStatus.WAIT_BUYER_PAY.getStatus().equals(tradeStatus)){
                        updateOrder(order,result,OrdersStatusEnum.UNPAYED.value());
                    }
                    log.error("没有处理支付宝回调业务，支付宝交易状态：{},params:{}",tradeStatus, paramsJson);
                }
                //释放锁
                distributedLocker.unlock(lockKey);
                break;
            }
        });
    }

    // 将request中的参数转换成Map
    private static Map<String, String> convertRequestParamsToMap(HttpServletRequest request) {
        Map<String, String> retMap = new HashMap();

        Set<Map.Entry<String, String[]>> entrySet = request.getParameterMap().entrySet();

        for (Map.Entry<String, String[]> entry : entrySet) {
            String name = entry.getKey();
            String[] values = entry.getValue();
            int valLen = values.length;

            if (valLen == 1) {
                retMap.put(name, values[0]);
            } else if (valLen > 1) {
                StringBuilder sb = new StringBuilder();
                for (String val : values) {
                    sb.append(",").append(val);
                }
                retMap.put(name, sb.substring(1));
            } else {
                retMap.put(name, "");
            }
        }

        return retMap;
    }

    private AlipayNotifyParam buildAlipayNotifyParam(Map<String, String> params) {
        String json = JSON.toJSONString(params);
        return JSON.parseObject(json, AlipayNotifyParam.class);
    }

    /**
     * 1、商户需要验证该通知数据中的out_trade_no是否为商户系统中创建的订单号，
     * 2、判断total_amount是否确实为该订单的实际金额（即商户订单创建时的金额），
     * 3、校验通知中的seller_id（或者seller_email)是否为out_trade_no这笔单据的对应的操作方（有的时候，一个商户可能有多个seller_id/seller_email），
     * 4、验证app_id是否为该商户本身。上述1、2、3、4有任何一个验证不通过，则表明本次通知是异常通知，务必忽略。
     * 在上述验证通过后商户必须根据支付宝不同类型的业务通知，正确的进行不同的业务处理，并且过滤重复的通知结果数据。
     * 在支付宝的业务通知中，只有交易通知状态为TRADE_SUCCESS或TRADE_FINISHED时，支付宝才会认定为买家付款成功。
     *
     * @param params params
     * @throws AlipayApiException 阿里支付异常
     */
    private void check(Map<String, String> params, AlipayConfig alipayConfig) throws AlipayApiException {
        String outTradeNo = params.get("out_trade_no");

        // 1、商户需要验证该通知数据中的out_trade_no是否为商户系统中创建的订单号，
        Orders order = getByTradeNo(outTradeNo);
        if (order == null) {
            throw new AlipayApiException("out_trade_no错误");
        }

        // 2、判断total_amount是否确实为该订单的实际金额（即商户订单创建时的金额），
        long totalAmount = new BigDecimal(params.get("total_amount")).longValue();
        if (totalAmount != order.getPrice().longValue()) {
            throw new AlipayApiException("error total_amount");
        }

        // 3、校验通知中的seller_id（或者seller_email)是否为out_trade_no这笔单据的对应的操作方（有的时候，一个商户可能有多个seller_id/seller_email），
        // 第三步可根据实际情况省略
        String sellerId = params.get("seller_id");
        if(!config.getProperty("ali.pay.sellerId").equals(sellerId)){
            throw new AlipayApiException("error seller_id");
        }

        // 4、验证app_id是否为该商户本身。
        if (!params.get("app_id").equals(alipayConfig.getAppId())) {
            throw new AlipayApiException("app_id不一致");
        }
    }

}
