package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.ClientEnum;
import cn.kdan.pdf.backend.core.enums.GoodsTypeEnum;
import cn.kdan.pdf.backend.core.enums.SaleStatusEnum;
import cn.kdan.pdf.backend.core.enums.SubscriberTypeEnum;
import cn.kdan.pdf.backend.core.mapper.PricingsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.model.pricing.QueryPricingReq;
import cn.kdan.pdf.backend.core.model.pricing.QueryPricingResp;
import cn.kdan.pdf.backend.core.model.pricing.PricingVo;
import cn.kdan.pdf.backend.core.params.QueryPricingParam;
import cn.kdan.pdf.backend.core.pojo.SetPricingVO;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.CommonBusinessUtils;
import enums.CommonEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class PricingServiceImpl implements PricingService {

    @Autowired
    private PricingsMapper pricingsMapper;
    @Autowired
    private PricingDiscountService pricingDiscountService;
    @Autowired
    private SubscriptionService subscriptionService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private SetPricingService setPricingService;

    /**
     * 1个月
     */
    private final static String ONE_MONTH = "1.month";

    /**
     * 查询券服务列表
     */
    @Override
    public QueryPricingResp getPricingList(QueryPricingReq req) {
        // 判断客户端
        String client = req.getClient();
        if (StringUtils.isEmpty(client)) {
            client = ClientEnum.WEB.value();
        }
        String language = req.getLanguage();
        language = CommonBusinessUtils.judgeLanguage(language);

        Members member = memberService.getCurrentUser();
        PricingsExample example = new PricingsExample();
        //获取在售的券服务(谷歌支付的产品为17pdf开头，人民币支付的产品不为17pdf开头)
        example.createCriteria().andSaleTypeEqualTo(Integer.parseInt(SaleStatusEnum.FOR_SALE.value())).andProductCodeNotLike("17pdf%");
        List<Pricings> pricingList = pricingsMapper.selectByExample(example);
        List<PricingVo> list = new ArrayList<>();
        for (Pricings pricings : pricingList) {
            PricingVo vo = new PricingVo();
            BeanUtils.copyProperties(pricings, vo);
            //如果语言是英文，设置口号为英文版，否则为中文版
            if ("en".equals(language)) {
                vo.setSlogan(pricings.getSloganEn());
            } else {
                vo.setSlogan(pricings.getSlogan());
            }
            PricingDiscounts activePricingDiscount = null;
            //填充折扣对象
            if (!StringUtils.isEmpty(pricings.getPricingDiscountId())) {
                //判断是否有活跃的折扣对象
                activePricingDiscount = getActiveDiscount(pricings.getPricingDiscountId(), member);
                vo.setPricingDiscounts(activePricingDiscount);
                vo.setAmount(countPointsAmount(pricings, activePricingDiscount));
                //设置时间
                vo.setPeriod(countPeriod(pricings, activePricingDiscount));
            }
            //计算真实价格
            if (!ObjectUtils.isEmpty(pricings.getMarketPrice()) && !ObjectUtils.isEmpty(pricings.getWebDiscounts())) {
                vo.setPrice(countPrice(pricings, activePricingDiscount, member, client));
            }
            //设置web折扣
            vo.setWebDiscounts(countDiscount(pricings, activePricingDiscount, member, ClientEnum.WEB.value()));
            //设置app折扣
            vo.setAppDiscounts(countDiscount(pricings, activePricingDiscount, member, ClientEnum.APP.value()));
            list.add(vo);
        }
        QueryPricingResp resp = new QueryPricingResp();
        resp.setPricingList(list);
        resp.setType(CommonEnum.DataTypeEnum.PRICINGS.value());
        return resp;
    }

    /**
     * 计算时间
     */
    @Override
    public String countPeriod(Pricings item, PricingDiscounts pricingDiscount) {
        //为空的时候不加额外的时间
        return ObjectUtils.isEmpty(pricingDiscount) ? item.getPeriod() : item.getPeriod() + pricingDiscount.getExtraPeriod();
    }

    @Override
    public Pricings getById(Integer id) {
        return pricingsMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<Pricings> getForSaleRMBById(String pricingId) {
        PricingsExample example = new PricingsExample();
        example.createCriteria().andSaleTypeEqualTo(Integer.parseInt(SaleStatusEnum.FOR_SALE.value())).andProductCodeNotLike("17pdf%").andIdEqualTo(Integer.parseInt(pricingId));
        return pricingsMapper.selectByExample(example);
    }

    /**
     * 获取活跃的折扣对象
     */
    private PricingDiscounts getActiveDiscount(String pricingDiscountId, Members member) {
        PricingDiscounts pricingDiscount = pricingDiscountService.getById(pricingDiscountId);
        //如果用户有活跃的折扣
        //计算折扣是否活跃，折扣次数是否用完
        //计算时间是否在期内
        Date startDate = pricingDiscount.getStartDate();
        Date endDate = pricingDiscount.getEndDate();
        boolean activeDate = (ObjectUtils.isEmpty(startDate) || startDate.compareTo(new Date()) < 0) && (ObjectUtils.isEmpty(endDate) || endDate.compareTo(new Date()) > 0);
        //计算可用次数
        boolean activeTimes = true;
        if (!ObjectUtils.isEmpty(member) && member.getSubscriberType() == 1) {
            activeTimes = false;
        } else if (!ObjectUtils.isEmpty(pricingDiscount.getLimitTimes()) && subscriptionService.countTimesById(member.getId(), pricingDiscount.getId()) > pricingDiscount.getLimitTimes()) {
            activeTimes = false;
        }
        return activeDate && activeTimes ? pricingDiscount : null;
    }

    /**
     * 计算折扣
     */
    private float countDiscount(Pricings item, PricingDiscounts activeDiscount, Members member, String client) {
        Integer subscriberType = ObjectUtils.isEmpty(member) ? SubscriberTypeEnum.no.value() : member.getSubscriberType();
        if (!isGooglePlay(item) && item.getPeriod().contains(ONE_MONTH) && SubscriberTypeEnum.yes.value().equals(subscriberType)) {
            return (float) 0.7;
        } else {
            if (!ObjectUtils.isEmpty(activeDiscount) && activeDiscount.getDiscounts() > 0) {
                return activeDiscount.getDiscounts();
            } else {
                return ClientEnum.APP.value().equals(client) ? item.getAppDiscounts() : item.getWebDiscounts();
            }
        }
    }

    private boolean isGooglePlay(Pricings item) {
        return item.getProductCode().contains("17pdf");
    }

    /**
     * 计算价格
     */
    @Override
    public float countPrice(Pricings item, PricingDiscounts activeDiscount, Members member, String client) {
        Integer subscriberType = ObjectUtils.isEmpty(member) ? SubscriberTypeEnum.no.value() : member.getSubscriberType();
        if (item.getPeriod().contains(ONE_MONTH) && SubscriberTypeEnum.yes.value().equals(subscriberType)) {
            return discountPrice(item, activeDiscount, member, client);
        } else if (ObjectUtils.isEmpty(activeDiscount)) {
            return discountPrice(item, activeDiscount, member, client);
        } else {
            if (activeDiscount.getPrice() > 0) {
                return activeDiscount.getPrice();
            } else if (activeDiscount.getDiscounts() > 0) {
                return discountPrice(item, activeDiscount, member, client);
            } else {
                return discountPrice(item, activeDiscount, member, client);
            }
        }
    }

    /**
     * 折扣价
     */
    private float discountPrice(Pricings item, PricingDiscounts activeDiscount, Members member, String client) {
        //TODO 保留一位小数
        return isGooglePlay(item) ? item.getMarketPrice() : (item.getMarketPrice() * countDiscount(item, activeDiscount, member, client));
    }

    /**
     * 计算剩余券数量
     */
    @Override
    public Integer countPointsAmount(Pricings item, PricingDiscounts pricingDiscount) {
        //为空的时候不加额外的券数
        return ObjectUtils.isEmpty(pricingDiscount) ? item.getAmount() : item.getAmount() + pricingDiscount.getExtraPoints();
    }

    @Override
    public QueryPricingResp appList(QueryPricingParam param) {
        QueryPricingResp resp = new QueryPricingResp();
        Members currentUser = memberService.getCurrentUser();
        QueryPricingReq req = new QueryPricingReq();
        String client = param.getClient();
        req.setClient(client);
        req.setLanguage(param.getLanguage());
        String type = param.getType();
        if(GoodsTypeEnum.Pricing.value().equals(type)){
            resp = getPricingList(req);
        }else{
            List<SetPricingVO> list = setPricingService.list(client, currentUser);
            resp.setSetPricingList(list);
            resp.setType(GoodsTypeEnum.SetPricing.value());
        }
        return resp;
    }
}
