package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.ClientEnum;
import cn.kdan.pdf.backend.core.enums.SaleStatusEnum;
import cn.kdan.pdf.backend.core.enums.SubscriberTypeEnum;
import cn.kdan.pdf.backend.core.mapper.SetPricingsMapper;
import cn.kdan.pdf.backend.core.mapper.ext.ExtSetPricingsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.pojo.SetPricingVO;
import cn.kdan.pdf.backend.core.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class SetPricingServiceImpl implements SetPricingService {

    @Autowired
    private SetPricingsMapper setPricingsMapper;
    @Autowired
    private PricingDiscountService pricingDiscountService;
    @Autowired
    private SubscriptionService subscriptionService;
    @Autowired
    private ExtSetPricingsMapper extSetPricingsMapper;

    @Override
    public List<SetPricingVO> list(String client,Members member) {
        if (StringUtils.isEmpty(client)) {
            client = ClientEnum.WEB.value();
        }
        SetPricingsExample example = new SetPricingsExample();
        //获取在售的会员类型
        example.createCriteria().andSaleTypeEqualTo(Integer.parseInt(SaleStatusEnum.FOR_SALE.value())).andProductCodeNotLike("17pdf%");
        List<SetPricings> list = setPricingsMapper.selectByExample(example);
        List<SetPricingVO> resultList = new ArrayList<>();
        String finalClient = client;
        list.forEach(item -> {
            SetPricingVO pojo = new SetPricingVO();
            BeanUtils.copyProperties(item, pojo);
            PricingDiscounts activePricingDiscount = null;
            //填充折扣对象
            if (!StringUtils.isEmpty(item.getPricingDiscountId())) {
                //判断是否有活跃的折扣对象
                activePricingDiscount = getActiveDiscount(item.getPricingDiscountId(), member);
                pojo.setPricingDiscount(activePricingDiscount);
                //如果有折扣，加上折扣中的额外券数
                pojo.setPointsAmount(countPointsAmount(item, activePricingDiscount));
                //设置时间
                pojo.setPeriod(countPeriod(item, activePricingDiscount));
            }
            //计算真实价格
            if (!ObjectUtils.isEmpty(item.getMarketPrice()) && !ObjectUtils.isEmpty(item.getWebDiscounts())) {
                pojo.setPrice(countPrice(item, activePricingDiscount, member, finalClient));
            }
            //设置web折扣
            pojo.setWebDiscounts(countDiscount(item, activePricingDiscount, member, ClientEnum.WEB.value()));
            //设置app折扣
            pojo.setAppDiscounts(countDiscount(item, activePricingDiscount, member, ClientEnum.APP.value()));
            resultList.add(pojo);
        });
        return resultList;
    }

    @Override
    public List<SetPricings> getForSaleRMBById(String setPricingId) {
        SetPricingsExample example = new SetPricingsExample();
        //获取在售的会员类型
        example.createCriteria().andSaleTypeEqualTo(Integer.parseInt(SaleStatusEnum.FOR_SALE.value())).andProductCodeNotLike("17pdf%").andIdEqualTo(Integer.parseInt(setPricingId));
        return setPricingsMapper.selectByExample(example);
    }

    @Override
    public SetPricings getById(Integer setPricingId) {
        return setPricingsMapper.selectByPrimaryKey(setPricingId);
    }

    /**
     * 获取活跃的折扣对象
     */
    @Override
    public PricingDiscounts getActiveDiscount(String pricingDiscountId, Members member) {
        PricingDiscounts pricingDiscount = pricingDiscountService.getById(pricingDiscountId);
        //如果用户有活跃的折扣
        //计算折扣是否活跃，折扣次数是否用完
        //计算时间是否在期内
        Date startDate = pricingDiscount.getStartDate();
        Date endDate = pricingDiscount.getEndDate();
        boolean activeDate = (ObjectUtils.isEmpty(startDate) || startDate.compareTo(new Date()) < 0) && (ObjectUtils.isEmpty(endDate) || endDate.compareTo(new Date()) > 0);
        //计算可用次数
        boolean activeTimes = true;
        if (!ObjectUtils.isEmpty(member) && member.getSubscriberType() == 1) {
            activeTimes = false;
        } else if (!ObjectUtils.isEmpty(pricingDiscount.getLimitTimes()) && subscriptionService.countTimesById(member.getId(), pricingDiscount.getId()) > pricingDiscount.getLimitTimes()) {
            activeTimes = false;
        }
        return activeDate && activeTimes ? pricingDiscount : null;
    }


    /**
     * 计算剩余券数量
     */
    @Override
    public Integer countPointsAmount(SetPricings item, PricingDiscounts pricingDiscount) {
        //为空的时候不加额外的券数
        return ObjectUtils.isEmpty(pricingDiscount) ? item.getPointsAmount() : item.getPointsAmount() + pricingDiscount.getExtraPoints();
    }

    /**
     * 计算时间
     */
    @Override
    public String countPeriod(SetPricings item, PricingDiscounts pricingDiscount) {
        //为空的时候不加额外的时间
        return ObjectUtils.isEmpty(pricingDiscount) ? item.getPeriod() : item.getPeriod() + pricingDiscount.getExtraPeriod();
    }

    /**
     * 计算价格
     */
    @Override
    public float countPrice(SetPricings item, PricingDiscounts activeDiscount, Members member, String client) {
        Integer subscriberType = ObjectUtils.isEmpty(member) ? SubscriberTypeEnum.no.value() : member.getSubscriberType();
        if (item.getPeriod().contains("1.month") && SubscriberTypeEnum.yes.value().equals(subscriberType)) {
            return discountPrice(item, activeDiscount, member, client);
        } else if (ObjectUtils.isEmpty(activeDiscount)) {
            return discountPrice(item, activeDiscount, member, client);
        } else {
            if (activeDiscount.getPrice() > 0) {
                return activeDiscount.getPrice();
            } else if (activeDiscount.getDiscounts() > 0) {
                return discountPrice(item, activeDiscount, member, client);
            } else {
                return discountPrice(item, activeDiscount, member, client);
            }
        }
    }

    /**
     * 折扣价
     */
    private float discountPrice(SetPricings item, PricingDiscounts activeDiscount, Members member, String client) {
        //TODO 保留一位小数
        return isGooglePlay(item) ? item.getMarketPrice() : (item.getMarketPrice() * countDiscount(item, activeDiscount, member, client));
    }

    /**
     * 计算折扣
     */
    private float countDiscount(SetPricings item, PricingDiscounts activeDiscount, Members member, String client) {
        Integer subscriberType = ObjectUtils.isEmpty(member) ? SubscriberTypeEnum.no.value() : member.getSubscriberType();
         if (!isGooglePlay(item) && item.getPeriod().contains("1.month") && SubscriberTypeEnum.yes.value().equals(subscriberType)) {
            return (float) 0.7;
        } else {
            if (!ObjectUtils.isEmpty(activeDiscount) && activeDiscount.getDiscounts() > 0) {
                return activeDiscount.getDiscounts();
            } else {
                return ClientEnum.APP.value().equals(client) ? item.getAppDiscounts() : item.getWebDiscounts();
            }
        }
    }

    private boolean isGooglePlay(SetPricings item) {
        return item.getProductCode().contains("17pdf");
    }


    @Override
    public List<Subscriptions> selectMode0(String memberId) {
        return extSetPricingsMapper.selectMode0(memberId);
    }

    @Override
    public List<Subscriptions> selectPByModeAndMemberId(String memberId, String mode) {
        return extSetPricingsMapper.selectByModeAndMemberId(memberId,mode);
    }
}
