package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.enums.ProviderEnum;
import cn.kdan.pdf.backend.core.enums.SubscriberTypeEnum;
import cn.kdan.pdf.backend.core.mapper.SocialAccountsMapper;
import cn.kdan.pdf.backend.core.model.Members;
import cn.kdan.pdf.backend.core.model.SocialAccounts;
import cn.kdan.pdf.backend.core.model.SocialAccountsExample;
import cn.kdan.pdf.backend.core.params.BindAccountParam;
import cn.kdan.pdf.backend.core.pojo.app.SocialAccountVo;
import cn.kdan.pdf.backend.core.pojo.third.Tencent;
import cn.kdan.pdf.backend.core.pojo.third.Wechat;
import cn.kdan.pdf.backend.core.service.MemberService;
import cn.kdan.pdf.backend.core.service.SocialAccountService;
import cn.kdan.pdf.backend.core.utils.TencentUtils;
import cn.kdan.pdf.backend.core.utils.WechatUtils;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author SongFuQiang
 * @description
 */
@Slf4j
@Service
public class SocialAccountServiceImpl implements SocialAccountService {

    @Resource
    private SocialAccountsMapper socialAccountsMapper;
    @Resource
    private MemberService memberService;

    @Override
    public SocialAccounts selectByAccountId(String accountId) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andAccountIdEqualTo(accountId);
        List<SocialAccounts> list = socialAccountsMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(list)) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public int insertSocialAccount(SocialAccounts socialAccounts) {
        Date date = new Date();
        socialAccounts.setCreatedAt(date);
        socialAccounts.setUpdatedAt(date);
        socialAccountsMapper.insert(socialAccounts);
        return 0;
    }

    @Override
    public List<SocialAccounts> selectByMemberId(String userId) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andMemberIdEqualTo(userId);
        return socialAccountsMapper.selectByExample(example);
    }

    public SocialAccounts selectByMemberIdAndProvider(String userId, Integer provider){
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andMemberIdEqualTo(userId).andProviderEqualTo(provider);
        List<SocialAccounts> list = socialAccountsMapper.selectByExample(example);
        if(!CollectionUtils.isEmpty(list)){
            return list.get(0);
        }
        return null;
    }

    @Override
    public void deleteByMemberId(String memberId) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andMemberIdEqualTo(memberId);
        socialAccountsMapper.deleteByExample(example);
    }

    @Override
    public List<SocialAccountVo> bindSocialAccount(Members member, BindAccountParam param, List<SocialAccountVo> socialAccounts) {
        String accessToken = param.getAccessToken();
        String openid = param.getOpenid();
        String provider = param.getProvider();
        if(ProviderEnum.TENCENT.getName().equals(provider)){
            Tencent userInfo = TencentUtils.getUserInfo(accessToken);
            String unionId = userInfo.getUnionId();
            if(StringUtils.isEmpty(unionId)){
                log.info("未找到第三方账号,accessToken:{},openid:{}",accessToken,openid);
                throw new BackendRuntimeException("未找到第三方账号");
            }
            SocialAccounts socialAccount = selectByAccountId(unionId);
            //看看有没有该账号，没有就新增一条，有就更新用户id
            if(ObjectUtils.isEmpty(socialAccount)){
                socialAccount = new SocialAccounts();
                socialAccount.setId(CommonUtils.generateId());
                socialAccount.setAccountId(unionId);
                socialAccount.setMemberId(member.getId());
                socialAccount.setName(userInfo.getNickname());
                socialAccount.setProvider(ProviderEnum.getCodeByName(provider));
                socialAccount.setAvatar(userInfo.getFigureUrl());
                insertSocialAccount(socialAccount);
            } else{
                //如果有第三方账号，需要先将原用户账号删除，再进行绑定当前用户账号
                String memberId = socialAccount.getMemberId();
                verifyMemberPayed(memberId);
                memberService.deleteByMemberId(memberId);
                socialAccount.setMemberId(member.getId());
                updateSocialAccount(socialAccount);
            }
        }else{
            Wechat userInfo = WechatUtils.getUserInfo(openid, accessToken);
            String unionId = userInfo.getUnionId();
            if(StringUtils.isEmpty(unionId)){
                log.info("未找到第三方账号,accessToken:{},openid:{}",accessToken,openid);
                throw new BackendRuntimeException("未找到第三方账号");
            }
            SocialAccounts socialAccount = selectByAccountId(unionId);
            if(ObjectUtils.isEmpty(socialAccount)){
                socialAccount = new SocialAccounts();
                socialAccount.setId(CommonUtils.generateId());
                socialAccount.setAvatar(userInfo.getHeadImgUrl());
                socialAccount.setAccountId(unionId);
                socialAccount.setProvider(ProviderEnum.getCodeByName(provider));
                socialAccount.setMemberId(member.getId());
                socialAccount.setName(userInfo.getNickname());
                insertSocialAccount(socialAccount);
            } else{
                //如果有第三方账号，需要先将原用户账号删除，再进行绑定当前用户账号
                String memberId = socialAccount.getMemberId();
                verifyMemberPayed(memberId);
                memberService.deleteByMemberId(memberId);
                socialAccount.setMemberId(member.getId());
                updateSocialAccount(socialAccount);
            }
        }
        List<SocialAccounts> list = selectByMemberId(member.getId());
        for (SocialAccounts account : list) {
            SocialAccountVo vo = new SocialAccountVo();
            vo.setProvider(ProviderEnum.getNameByCode(account.getProvider()));
            vo.setName(account.getName());
            vo.setAccountId(account.getAccountId());
            socialAccounts.add(vo);
        }
        return socialAccounts;
    }

    /**
     * 判断用户是否付费过
     * @param memberId 用户id
     */
    public void verifyMemberPayed(String memberId){
        Members member = memberService.getById(memberId);
        if(member.getTotalPoints() > 10 || SubscriberTypeEnum.yes.value().equals(member.getSubscriberType())){
            throw new BackendRuntimeException("不允许绑定该账号，因为充过钱了！");
        }
    }

    /**
     * 根据accountId更新社交账号
     * @param socialAccount 社交账号信息
     */
    public void updateSocialAccount(SocialAccounts socialAccount) {
        SocialAccountsExample example = new SocialAccountsExample();
        example.createCriteria().andAccountIdEqualTo(socialAccount.getAccountId());
        socialAccount.setUpdatedAt(new Date());
        socialAccountsMapper.updateByExample(socialAccount,example);
    }

    @Override
    public List<SocialAccountVo> unbindSocialAccount(Members currentUser, String provider, List<SocialAccountVo> accounts) {
        SocialAccountsExample example = new SocialAccountsExample();
        SocialAccounts socialAccount = selectByMemberIdAndProvider(currentUser.getId(), ProviderEnum.getCodeByName(provider));
        example.createCriteria().andAccountIdEqualTo(socialAccount.getAccountId());
        socialAccountsMapper.deleteByExample(example);
        List<SocialAccounts> list = selectByMemberId(currentUser.getId());
        for (SocialAccounts account : list) {
            SocialAccountVo vo = new SocialAccountVo();
            vo.setProvider(ProviderEnum.getNameByCode(account.getProvider()));
            vo.setName(account.getName());
            vo.setAccountId(account.getAccountId());
            accounts.add(vo);
        }

        return accounts;
    }
}
